/*
 * Copyright (c) 1996, 1997, 1998
 *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	$Id: nis_cache_bind.x,v 1.2 1999/02/09 15:33:08 wpaul Exp wpaul $
 */

#ifndef RPC_HDR
%#ifndef lint
%static const char rcsid[] = "@(#) $Id: nis_cache_bind.x,v 1.2 1999/02/09 15:33:08 wpaul Exp wpaul $";
%#endif
#endif

#ifdef RPC_HDR
%#include <rpcsvc/nis.h>
#endif

/*
 * This file describes the layout of the FreeBSD NIS+ cache file.
 * This information is needed by both the NIS+ client library code
 * and nis_cachemgr(8), so it needs to be somewhere from which it
 * can be easily shared. The client library contains XDR filters
 * generated using this protocol definition.
 *
 * The RPC interface to nis_cachemgr is specified in a different file
 * (nis_cache.x) which contains a protocol specified by Sun. This is
 * only half of the interface to nis_cachemgr though: the other half
 * is the NIS_SHARED_DIRCACHE file format, which is not accessible
 * directly through the NIS+ API. Since there are no public interfaces,
 * Sun has kept their cache implementation proprietary. This requires
 * us to define our own interface for this implimentation.
 */

const CB_CHUNKSIZE		= 8192;
const CB_HEADER_MAGIC		= 987654321;
const CB_VERSION		= 2;

enum cb_type {
	CB_MASTER		= 0,
	CB_REPLICA		= 1,
	CB_SERVER		= 2
};

/*
 * We could actually use a discriminated union here,
 * but the XDR encoding ends up being about the same either
 * way, and rpcgen can create really wacky member names when
 * using unions, so we may as well keep it simple.
 */
struct cachebind_hash_entry {
	unsigned long			hval;
	unsigned long			cb_type;
	nis_name			name;
	nis_server			*srv_ptr;
	nis_bound_endpoint		*bep_ptr;
	nis_bound_directory		*bdr_ptr;
	nis_active_endpoint		*aep_ptr;
	unsigned long			cr_time;
	unsigned long			next;
};

const TABLESIZE		= 256;
const HASH_MASK		= 0x000000FF;

typedef cachebind_hash_entry *cachebind_hash_entry_p;

enum cb_state {
	CB_UPDATING		= 0,
	CB_LOCKED		= 1,
	CB_UNLOCKED		= 2,
	CB_BINDING		= 3,
	CB_STABLE		= 4
};

struct cachebind_header {
	unsigned long		cb_magic;
	unsigned long		cb_vers;
	cb_state		cb_state;
	unsigned long		cb_generation;
	unsigned long		cb_starttime;
	unsigned long		cb_updatetime;
};

struct cachebind_table {
	cachebind_header	cb_header;
	unsigned long		cb_table[TABLESIZE];
};

struct cachebind_combined {
	cachebind_header	cachebind_header;
	cachebind_hash_entry_p	cachebind_hash_table[TABLESIZE];
};
