/*
 * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify Sun RPC without charge, but are not authorized
 * to license or distribute it to anyone else except as part of a product or
 * program developed by the user.
 * 
 * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 * 
 * Sun RPC is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 * 
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
 * OR ANY PART THEREOF.
 * 
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even if
 * Sun has been advised of the possibility of such damages.
 * 
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */

/*
 * Copyright (c) 1991, Sun Microsystems Inc.
 */

/*
 * This file contains the interfaces that are visible in the SunOS 5.x
 * implementation of NIS Plus. When using C++ the defined __cplusplus and
 * __STDC__ should both be true.
 */

#ifndef	_RPCSVC_NISLIB_H
#define	_RPCSVC_NISLIB_H

/* From: #pragma ident	"@(#)nislib.h	1.16	94/05/03 SMI" */

#ifdef __cplusplus
extern "C" {
#endif

struct signature {
	int signature_len;
	char *signature_val;
};

#define BTYPE_NAME	0
#define BTYPE_DIR	1
#define BTYPE_SRV	2

#define NIS_FREE_BINDDATA	0
#define NIS_KEEP_BINDDATA	1

struct nis_binding {
	CLIENT			*clnt;		/* RPC CLIENT handle */
	unsigned long		flags;		/* Flags */
	directory_obj		*dir;		/* Directory */
	nis_server		*srv;		/* Explicitly chosen server */
	unsigned long		cur;		/* Current server index */
	int			srv_idx;
	int			ep_idx;
	unsigned long		btype;
	int			freeact;
	netobj			*pkey;		/* Server's public key */
	struct sockaddr_in	srv_addr;	/* Server's IP address */
	int			srv_sock;	/* Server's local socket */
	unsigned short		srv_port;	/* Local port */
	time_t			ctime;		/* Binding creation time */
	int			refcnt;		/* Reference count */
};


#define NB_TYPE_CACHED		1
#define NB_TYPE_MANUAL		2

#define MB_MASTER		0
#define MB_REPLICA		1
#define MB_SERVER		2

struct manual_binding {
	int			mb_type; /* Binding type */
	directory_obj		*mb_dir; /* Directory object. */
	nis_server		*mb_srv; /* Server object */
	int			mb_srv_idx;/* Server index (if dir is present)*/
	int			mb_ep_idx; /* Current Endpoint index. */
};

struct _nis_binding {
	/*
	 * Binding name. This is the name of the object
	 * for which this binding applies. This may be a
	 * directory or server name.
	 */
	nis_name		nb_name;
	/*
	 * Type of binding. CACHE bindings come from
	 * the cache manager. MANUAL bindings are
	 * generated on the fly.
	 */
	int			nb_type;

	/* A _copy_ of binding info if this is a CACHE binding.
	 * Note that this is really an cachebind_hash_entry structure;
	 * a void pointer is used here to avoid polluting this header
	 * by #including the cachebind header, which adds other
	 * dependencies that the non-caching NIS+ client code should
	 * not have to deal with.
	 */
	void		*nb_binding_cached;

	/* A _copy_ of binding info if this is a MANUAL binding */
	struct manual_binding	*nb_binding_manual;

	/*
	 * Generic per-binding connection info. This includes
	 * the RPC client handle, public key of the
	 * guy on the other side, reference count, local port.
	 */
	CLIENT			*nb_clnt;
	unsigned long		nb_flags; /* DG, VC, AUTH */
	int			nb_sock;
	netobj			*nb_pkey;
	int			nb_refcnt;
	struct sockaddr_in	nb_srv_addr;
	unsigned short		nb_port;  /* local port */
};


#define CLIENT_CLOCK	&ct
#define SERVER_CLOCK	&zt
#define CACHE_CLOCK	&at
#define DATABASE_CLOCK	&dt

#define CLIENT_CLOCK_DECLS	struct timeval ct = { 0, 0 }
#define SERVER_CLOCK_DECLS	struct timeval zt = { 0, 0 }
#define CACHE_CLOCK_DECLS	struct timeval at = { 0, 0 }
#define DATABASE_CLOCK_DECLS	struct timeval dt = { 0, 0 }

#ifdef __STDC__
extern void nis_freeresult(nis_result *);
extern nis_result * nis_lookup(nis_name, u_long);
extern nis_result * nis_list(nis_name, u_long,
	int (*)(nis_name, nis_object *, void *), void *);
extern nis_result * nis_add(nis_name, nis_object *);
extern nis_result * nis_remove(nis_name, nis_object *);
extern nis_result * nis_modify(nis_name, nis_object *);

extern nis_result * nis_add_entry(nis_name, nis_object *, u_long);
extern nis_result * nis_remove_entry(nis_name, nis_object *, u_long);
extern nis_result * nis_modify_entry(nis_name, nis_object *, u_long);
extern nis_result * nis_first_entry(nis_name);
extern nis_result * nis_next_entry(nis_name, netobj *);

extern nis_error nis_mkdir(nis_name, nis_server *);
extern nis_error nis_rmdir(nis_name, nis_server *);
extern name_pos nis_dir_cmp(nis_name, nis_name);

extern nis_name * nis_getnames(nis_name);
extern void nis_freenames(nis_name *);
extern nis_name nis_domain_of(nis_name);
extern nis_name nis_leaf_of(nis_name);
extern nis_name nis_leaf_of_r(const nis_name, char *, size_t);
extern nis_name nis_name_of(nis_name);
extern nis_name nis_local_group(void);
extern nis_name nis_local_directory(void);
extern nis_name nis_local_principal(void);
extern nis_name nis_local_host(void);

extern void nis_destroy_object(nis_object *);
extern nis_object * nis_clone_object(nis_object *, nis_object *);
extern void nis_print_object(nis_object *);

extern char * nis_sperrno(nis_error);
extern void nis_perror(nis_error, char *);
extern char * nis_sperror(nis_error, char *);
extern void nis_lerror(nis_error, char *);

extern void nis_print_group_entry(nis_name);
extern bool_t nis_ismember(nis_name, nis_name);
extern nis_error nis_creategroup(nis_name, u_long);
extern nis_error nis_destroygroup(nis_name);
extern nis_error nis_addmember(nis_name, nis_name);
extern nis_error nis_removemember(nis_name, nis_name);
extern nis_error nis_verifygroup(nis_name);

extern void nis_freeservlist(nis_server **);
extern nis_server ** nis_getservlist(nis_name);
extern nis_error nis_stats(nis_server *, nis_tag *, int, nis_tag **);
extern nis_error nis_servstate(nis_server *, nis_tag *, int, nis_tag **);
extern void nis_freetags(nis_tag *, int);

extern nis_result * nis_checkpoint(nis_name);
extern void nis_ping(nis_name, u_long, nis_object *);

/*
 * XXX: PLEASE NOTE THAT THE FOLLOWING FUNCTIONS ARE INTERNAL
 * TO NIS+ AND SHOULD NOT BE USED BY ANY APPLICATION PROGRAM.
 * THEIR SEMANTICS AND/OR SIGNATURE CAN CHANGE WITHOUT NOTICE.
 * SO, PLEASE DO NOT USE THEM.  YOU ARE WARNED!!!!
 */

extern char ** __break_name(nis_name, int *);
extern int __name_distance(char **, char **);
extern nis_result * nis_make_error(nis_error, u_long, u_long, u_long, u_long);
extern nis_attr * __cvt2attr(int *, char **);
extern void nis_free_request(ib_request *);
extern nis_error nis_get_request(nis_name, nis_object *, netobj*, ib_request*);
extern nis_object * nis_read_obj(char *);
extern int nis_write_obj(char *, nis_object *);
extern int nis_in_table(nis_name, NIS_HASH_TABLE *, int *);
extern int nis_insert_item(NIS_HASH_ITEM *, NIS_HASH_TABLE *);
extern NIS_HASH_ITEM * nis_find_item(nis_name, NIS_HASH_TABLE *);
extern NIS_HASH_ITEM * nis_remove_item(nis_name, NIS_HASH_TABLE *);
extern void nis_insert_name(nis_name, NIS_HASH_TABLE *);
extern void nis_remove_name(nis_name, NIS_HASH_TABLE *);
extern CLIENT * nis_make_rpchandle(nis_server *, int, u_long, u_long, u_long,
								int, int);
extern void * nis_get_static_storage(struct nis_sdata *, u_long, u_long);
extern char * nis_data(char *);
extern void nis_print_rights(u_long);
extern void nis_print_directory(directory_obj *);
extern void nis_print_group(group_obj *);
extern void nis_print_table(table_obj *);
extern void nis_print_link(link_obj *);
extern void nis_print_entry(entry_obj *);
extern nis_object * nis_get_object(char *, char *, char *, u_long, u_long,
								    zotypes);
extern nis_server * __nis_init_callback(CLIENT *,
    int (*)(nis_name, nis_object *, void *), void *);
extern int nis_getdtblsize(void);
extern int __nis_run_callback(netobj *, u_long, struct timeval *, CLIENT *);

extern log_result *nis_dumplog(nis_server *, nis_name, u_long);
extern log_result *nis_dump(nis_server *, nis_name,
    int (*)(nis_name, nis_object *, void *));

extern bool_t __do_ismember(nis_name, nis_name,
    nis_result *(*)(nis_name, u_long));
extern nis_name __nis_map_group(nis_name);
extern nis_name __nis_map_group_r(nis_name, char*, size_t);

extern nis_error __nis_CacheBind(char *, directory_obj *);
extern nis_error __nis_CacheSearch(char *, directory_obj *);
extern bool_t __nis_CacheRemoveEntry(directory_obj *);
extern void __nis_CacheRestart(void);
extern void __nis_CachePrint(void);
extern void __nis_CacheDumpStatistics(void);
extern bool_t writeColdStartFile(directory_obj *);

extern CLIENT * __get_ti_clnt(char *, CLIENT *, char **, pid_t *);
extern int __strcmp_case_insens(char *, char *);
extern int __strncmp_case_insens(char *, char *);

extern fd_result * nis_finddirectory(directory_obj *, nis_name);
extern fd_result * nis_finddirectory_core(directory_obj *, nis_name);

/*
 * These particular functions are part of the FreeBSD NIS+ implementation
 * only. Ideally they should be somewhere else, but they are used by both
 * the library and some of the programs, and there aren't that many headers
 * common to both.
 */

extern int __start_clock(struct timeval *);
extern u_long __stop_clock(struct timeval *);

extern struct signature *__nis_calculate_encrypted_cksum(unsigned char *, unsigned int, char *, int);
extern unsigned long __nis_hash(const void *, size_t len);
extern nis_server *__nis_choose_server (struct _nis_binding *, unsigned long);
extern nis_server *__nis_choose_server_core (struct _nis_binding *,
						unsigned long);
extern nis_error __nis_find_server (nis_name, unsigned long, directory_obj **);
extern nis_error __nis_connect_server (struct _nis_binding *);
extern nis_error __nis_create_binding ( directory_obj *, nis_server *,
					unsigned long, struct _nis_binding **);
extern void __nis_destroy_binding (struct _nis_binding *);
extern nis_error __nis_bind (nis_name, unsigned long, struct _nis_binding **);
extern nis_error __nis_rebind (struct _nis_binding *, unsigned long);
extern void __nis_flush_current_binding (void);
extern nis_error __nis_release_binding (struct _nis_binding *);

extern nis_error nis_get_cold_start(directory_obj **);
extern nis_error nis_put_cold_start(directory_obj *);
extern nis_error __nis_uaddr_to_sockaddr(endpoint *, struct sockaddr_in *);
extern char *__nis_getpkey(char *, char *);
extern directory_obj *__nis_copy_dir(directory_obj *);
extern nis_server *__nis_copy_srv(nis_server *);

extern nis_error __nis_cache_add(fd_result *);
extern directory_obj *__nis_cache_check(nis_name);
extern void __nis_cache_close(void);
extern nis_error __nis_cache_refresh(nis_name);
extern nis_error __nis_cache_remove(directory_obj *);

extern void nis_flushgroups(void);

#else

/* Non-prototype definitions (old fashioned C) */

extern void nis_freeresult();
extern nis_result * nis_lookup();
extern nis_result * nis_list();
extern nis_result * nis_add();
extern nis_result * nis_remove();
extern nis_result * nis_modify();

extern nis_result * nis_add_entry();
extern nis_result * nis_remove_entry();
extern nis_result * nis_modify_entry();
extern nis_result * nis_first_entry();
extern nis_result * nis_next_entry();

extern nis_error nis_mkdir();
extern nis_error nis_rmdir();
extern name_pos nis_dir_cmp();

extern nis_name *nis_getnames();
extern void nis_freenames();
extern nis_name nis_domain_of();
extern nis_name nis_leaf_of();
extern nis_name nis_leaf_of_r();
extern nis_name nis_name_of();
extern nis_name nis_local_group();
extern nis_name nis_local_directory();
extern nis_name nis_local_principal();
extern nis_name nis_local_host();

extern void nis_destroy_object();
extern nis_object * nis_clone_object();
extern void nis_print_object();

extern char * nis_sperrno();
extern void nis_perror();
extern char * nis_sperror();
extern void nis_lerror();

extern void nis_print_group_entry();
extern bool_t nis_ismember();
extern nis_error nis_creategroup();
extern nis_error nis_destroygroup();
extern nis_error nis_addmember();
extern nis_error nis_removemember();
extern nis_error nis_verifygroup();

extern void nis_freeservlist();
extern nis_server ** nis_getservlist();
extern nis_error nis_stats();
extern nis_error nis_servstate();
extern void nis_freetags();

extern nis_result * nis_checkpoint();
extern void nis_ping();

/*
 * XXX: PLEASE NOTE THAT THE FOLLOWING FUNCTIONS ARE INTERNAL
 * TO NIS+ AND SHOULD NOT BE USED BY ANY APPLICATION PROGRAM.
 * THEIR SEMANTICS AND/OR SIGNATURE CAN CHANGE WITHOUT NOTICE.
 * SO, PLEASE DO NOT USE THEM.  YOU ARE WARNED!!!!
 */
extern char ** __break_name();
extern int __name_distance();
extern nis_result * nis_make_error();
extern nis_attr * __cvt2attr();
extern void nis_free_request();
extern nis_error nis_get_request();
extern nis_object * nis_read_obj();
extern int nis_write_obj();
extern int nis_in_table();
extern int nis_insert_item();
extern NIS_HASH_ITEM * nis_find_item();
extern NIS_HASH_ITEM * nis_remove_item();
extern void nis_insert_name();
extern void nis_remove_name();
extern CLIENT * nis_make_rpchandle();
extern void * nis_get_static_storage();
extern char * nis_data();

extern void nis_print_rights();
extern void nis_print_directory();
extern void nis_print_group();
extern void nis_print_table();
extern void nis_print_link();
extern void nis_print_entry();
extern nis_object * nis_get_object();

extern nis_server * __nis_init_callback();
extern int nis_getdtblsize();
extern int __nis_run_callback();

extern log_result * nis_dump();
extern log_result * nis_dumplog();

extern bool_t __do_ismember();
extern nis_name __nis_map_group();
extern nis_name __nis_map_group_r();


extern nis_error __nis_CacheBind();
extern directory_obj * __nis_CacheSearch();
extern bool_t __nis_CacheRemoveEntry();
extern void __nis_CacheRestart();
extern void __nis_CachePrint();
extern void __nis_CacheDumpStatistics();
extern bool_t writeColdStartFile();

extern CLIENT * __get_ti_clnt();
extern int __strcmp_case_insens();
extern int __strncmp_case_insens();

extern fd_result * nis_finddirectory();
extern fd_result * nis_finddirectory_core();

/*
 * These particular functions are part of the FreeBSD NIS+ implementation
 * only. Ideally they should be somewhere else, but they are used by both
 * the library and some of the programs, and there aren't that many headers
 * common to both.
 */

extern int __start_clock();
extern u_long __stop_clock();
extern struct signature *__nis_calculate_encrypted_cksum();
extern unsigned long __nis_hash();
extern nis_server *__nis_choose_server();
extern nis_server *__nis_choose_server_core();
extern nis_error __nis_find_server();
extern nis_error __nis_connect_server();
extern nis_error __nis_create_binding();
extern void __nis_destroy_binding();
extern nis_error __nis_bind();
extern nis_error __nis_rebind();
extern void __nis_flush_current_binding ();
extern nis_error __nis_release_binding ();

extern nis_error nis_get_cold_start();
extern nis_error nis_put_cold_start();
extern nis_error __nis_uaddr_to_sockaddr();
extern char *__nis_getpkey();
extern nis_server *__nis_copy_srv();

extern nis_error __nis_cache_add();
extern directory_obj *__nis_cache_check();
extern void __nis_cache_close();
extern nis_error __nis_cache_refresh();
extern nis_error __nis_cache_remove();
extern void nis_flushgroups();
#endif

#define	NUL '\0'

#ifdef	__cplusplus
}
#endif

#endif	/* _RPCSVC_NISLIB_H */
