/*
 * Copyright (c) 1996
 *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD$
 */

#include <stdlib.h>
#include <sys/types.h>
#include <limits.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <rpcsvc/nis.h>
#include <rpcsvc/nis_db.h>
#include "nis_db_struct.h"
#include "nis_db_private.h"

#ifndef lint
__RCSID("@(#) $FreeBSD$");
#endif

db_status db_create_table(table_name, table)
	char			*table_name;
	table_obj		*table;
{
	uint32_t		i;
	struct nis_dbent	*dptr;
	db_status		rval = DB_BADQUERY;
	uint32_t		*hashes;
	table_desc		tdesc;

	if (db_table_exists(table_name) == DB_SUCCESS)
		return(DB_NOTUNIQUE);

	/*
	 * A table name is not a file name. Don't allow fully
	 * qualified paths (this should also be enforced by the
	 * server, but be paranoid anyway.)
	 */

	if (strchr(table_name, '/'))
		return(DB_BADQUERY);

	/*
	 * Check that at least one column is searchable.
	 */

#define	NAME	table->ta_cols.ta_cols_val[i].tc_name

	hashes = (uint32_t *)calloc(ULONGSZ, table->ta_cols.ta_cols_len);
	for (i = 0; i < table->ta_cols.ta_cols_len; i++) {
		if (table->ta_cols.ta_cols_val[i].tc_flags & TA_SEARCHABLE) {
			rval = DB_SUCCESS;
			hashes[i] = hash(NAME,strlen(NAME),HASH_NORM);
		}
	}

	if (rval != DB_SUCCESS) {
		free(hashes);
		return(rval);
	}

	bzero((char *)&tdesc, sizeof(tdesc));

	tdesc.table = table;
	tdesc.table_name = table_name;
	tdesc.vers = known_versions[0].val;
	tdesc.index_hashes.index_hashes_val = (char *)hashes;
	tdesc.index_hashes.index_hashes_len =
	    table->ta_cols.ta_cols_len * ULONGSZ;

	if ((rval = update_catalog(table_name, &tdesc,
	    CAT_ADD)) != DB_SUCCESS) {
		free(hashes);
		return(rval);
	}

	if ((rval = cache_table(table_name, &tdesc)) != DB_SUCCESS) {
		free(hashes);
		return(rval);
	}

	if ((dptr = find_table(table_name)) == NULL) {
		free(hashes);
		return(DB_INTERNAL_ERROR);
	}

	dptr->index_hashes = hashes;

	return(DB_SUCCESS);
}
