/*
 * Copyright (c) 1996
 *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD$
 */

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <rpc/rpc.h>
#include <rpc/xdr.h>
#include <rpcsvc/nis.h>
#include <rpcsvc/nis_db.h>
#include "nis_db_private.h"

#ifndef lint
__RCSID("@(#) $FreeBSD$");
#endif

/*
 * This hash function is stolen directly from the
 * Berkeley DB package. It already exists inside libc, but
 * it's declared static which prevents us from calling it
 * from here.
 */

/*
 * OZ's original sdbm hash
 */
u_int32_t
hash(keyarg, len, u)
	const void		*keyarg;
	register size_t		len;
	int u;
{
	const u_char		*key;
	size_t			loop;
	u_int32_t		h;

/*#define HASHC   h = *key++ + 65599 * h */
#define HASHC   h = (u ? toupper(*key++) : *key++) + 65599 * h

	h = 0;
	key = keyarg;
	if (len > 0) {
		loop = (len + 8 - 1) >> 3;

		switch (len & (8 - 1)) {
		case 0:
			do {
				HASHC;
				/* FALLTHROUGH */
		case 7:
				HASHC;
				/* FALLTHROUGH */
		case 6:
				HASHC;
				/* FALLTHROUGH */
		case 5:
				HASHC;
				/* FALLTHROUGH */
		case 4:
				HASHC;
				/* FALLTHROUGH */
		case 3:
				HASHC;
				/* FALLTHROUGH */
		case 2:
				HASHC;
				/* FALLTHROUGH */
		case 1:
				HASHC;
			} while (--loop);
		}
	}
	return (h);
}

char *encode_table_desc(tdesc, s)
	table_desc		*tdesc;
	uint32_t		*s;
{
	uint32_t		size;
	char			*addr;
	XDR			xdrs;

	size = xdr_sizeof((xdrproc_t)xdr_table_desc, tdesc);

	if ((addr = calloc(1, size)) == NULL)
		return(NULL);
	xdrmem_create(&xdrs, addr, size, XDR_ENCODE);
	if (!xdr_table_desc(&xdrs, tdesc)) {
		xdr_destroy(&xdrs);
		free(addr);
		return(NULL);
	}
	xdr_destroy(&xdrs);

	*s = size;
	/* Note: must be free()'ed later! */
	return(addr);
}

/*
 * Decodes an XDR'ed buffer into table_desc form.
 */
table_desc *decode_table_desc(addr, size)
	char			*addr;
	uint32_t		size;
{
	XDR			xdrs;
	table_desc		*tdesc;

	if ((tdesc = calloc(1, sizeof(table_desc))) == NULL)
		return(NULL);
	xdrmem_create(&xdrs, addr, size, XDR_DECODE);
	if (!xdr_table_desc(&xdrs, tdesc)) {
		xdr_destroy(&xdrs);
		free(tdesc);
		return(NULL);
	}
	xdr_destroy(&xdrs);

	/* Note: must be free()'ed later! */
	return(tdesc);
}
