/*
 * Copyright (c) 1996
 *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD$
 */

#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <limits.h>
#include <rpcsvc/nis.h>
#include <rpcsvc/nis_db.h>
#include "nis_db_private.h"

#ifndef lint
__RCSID("@(#) $FreeBSD$");
#endif

db_result *db_next_entry(table_name, next_handle)
	char			*table_name;
	db_next_desc		*next_handle;
{
	db_result		*res, *rres;
	struct timeval		t_start, t_end;
	struct nis_dbent	*dptr = NULL;
	struct nis_nextinfo	*nextp = NULL;
	unsigned char		cksum;
	entry_obj		**entry;
	unsigned long		j, rval = DB_SUCCESS, *winners = NULL;

	gettimeofday(&t_start, NULL);

	if ((res = calloc(1, sizeof(db_result))) == NULL)
		return(NULL);

	if (next_handle == NULL || !next_handle->db_next_desc_len ||
		next_handle->db_next_desc_val == NULL) {
		res->status = DB_BADQUERY;
		goto bail;
	}

	if ((dptr = find_table(table_name)) == NULL) {
		res->status = DB_NOTFOUND;
		goto bail;
	}

	if (!next_handle->db_next_desc_len) {
		res->status = DB_BADQUERY;
		goto bail;
	}

	nextp = (struct nis_nextinfo *)next_handle->db_next_desc_val;
	cksum = nextp->cksum;
	nextp->cksum = NEXTINFO_MAGIC;
	if (__nis_db_cksum((unsigned char *)nextp,
		next_handle->db_next_desc_len) != cksum) {
		res->status = DB_BADQUERY;
		goto bail;
	}

	if (nextp->etype == ETYPE_BATCH) {
		caddr_t			np;
		np = (caddr_t)nextp;
		winners = (unsigned long *)(np + sizeof(struct nis_nextinfo));
		if (nextp->nextcount == *winners) {
			rres = db_reset_next_entry(table_name, next_handle);
			if (rres == NULL)
				res->status = DB_MEMORY_LIMIT;
			else {
				if (rres->status != DB_SUCCESS)
					res->status = rres->status;
				else
					res->status = DB_NOTFOUND;
				db_free_result(rres);
			}
			goto bail;
		}

		if ((entry = calloc(1, sizeof(entry_obj *))) == NULL) {
			res->status = DB_MEMORY_LIMIT;
			goto bail;
		}

		if ((*entry = calloc(1, sizeof(entry_obj))) == NULL) {
			res->status = DB_MEMORY_LIMIT;
			free(entry);
			goto bail;
		}

		for (j = nextp->nextcount + 1; j < winners[0]; j++) {
			rval = xff_get_entry(dptr, *entry, winners[j+1]);
			nextp->nextcount++;
			if (rval == DB_NOTFOUND)
				continue;
			else
				break;
		}

		
		if (rval != NIS_SUCCESS) {
			free(*entry);
			free(entry);
			if (rval == DB_NOTFOUND)
				res->status = DB_NOTFOUND;
			else
				res->status = DB_INTERNAL_ERROR;
			goto bail;
		}

		res->objects.objects_len = 1;
		res->objects.objects_val = entry;
		res->nextinfo.db_next_desc_len = next_handle->db_next_desc_len;
		res->nextinfo.db_next_desc_val = next_handle->db_next_desc_val;
		res->status = DB_SUCCESS;
		goto bail;
	}

	if ((entry = calloc(1, sizeof(entry_obj *))) == NULL) {
		res->status = DB_MEMORY_LIMIT;
		goto bail;
	}

	if ((*entry = calloc(1, sizeof(entry_obj))) == NULL) {
		res->status = DB_MEMORY_LIMIT;
		free(entry);
		goto bail;
	}

	if (nextp->pos >= dptr->hdr.table_size) {
		if (dptr->fp_log != NULL) {
		/* We're right at the end: move to the log, if any. */
			if (nextp->pos == dptr->hdr.table_size)
				fseek(dptr->fp_log,
				    dptr->hdr_log.start_of_entries,
				    SEEK_SET);
			else
				fseek(dptr->fp_log,
				    nextp->pos - dptr->hdr.table_size,
				    SEEK_SET);
		}
		if (dptr->fp != NULL)
			fseek(dptr->fp, dptr->hdr.table_size, SEEK_SET);
	} else {
		if (dptr->fp != NULL)
			fseek(dptr->fp, nextp->pos, SEEK_SET);
		if (dptr->fp_log != NULL)
			fseek(dptr->fp_log,
			    dptr->hdr_log.start_of_entries,
			    SEEK_SET);
	}

	if ((rval = xff_next_entry(dptr, *entry)) != DB_SUCCESS) {
		free(*entry);
		free(entry);
		if (rval == DB_NOTFOUND) {
			res->status = DB_NOTFOUND;
			rres = db_reset_next_entry(table_name, next_handle);
			if (rres == NULL)
				res->status = DB_MEMORY_LIMIT;
			else {
				if (rres->status != DB_SUCCESS)
					res->status = rres->status;
				else
					res->status = DB_NOTFOUND;
				db_free_result(rres);
			}
		} else
			res->status = DB_INTERNAL_ERROR;
		goto bail;
	}

	res->objects.objects_len = 1;
	res->objects.objects_val = entry;
	res->nextinfo.db_next_desc_len = next_handle->db_next_desc_len;
	res->nextinfo.db_next_desc_val = next_handle->db_next_desc_val;

bail:

	if (res->status == DB_SUCCESS) {
		if (nextp->etype == ETYPE_ENUM)
			nextp->pos = dptr->pos;
		nextp->cksum = NEXTINFO_MAGIC;
		nextp->cksum = __nis_db_cksum((unsigned char *)nextp,
		    res->nextinfo.db_next_desc_len);
	}

	next_handle->db_next_desc_len = 0;
	next_handle->db_next_desc_val = NULL;

	gettimeofday(&t_end, NULL);
	res->ticks += 
	    (t_end.tv_usec - t_start.tv_usec) +
	    ((t_end.tv_sec - t_start.tv_sec) * 1000000);

	return(res);
}
