/*
 * Copyright (c) 1996
 *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD$
 */

/*
 * This file describes interfaces and structures internal to the
 * library. While some of these are global symbols, they should never
 * be called by any code outside the library proper.
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <sys/cdefs.h>
#include <rpcsvc/nis.h>
#include "nis_db_struct.h"

#ifndef lint
static const char rcsid_db_private_h[] =
  "@(#) $FreeBSD$";
#endif

#define PERM_SECURE (S_IRUSR|S_IWUSR)

#define WFLAGS O_RDWR|O_CREAT
#define RFLAGS O_RDONLY

#define T_NOPOS 0
#define T_POS 1

#define CAT_ADD 0
#define CAT_DEL 1

#define HASH_UPPER 1
#define HASH_NORM  0

#define STATUS_NORMAL 0
#define STATUS_REPLAYING_LOG 1
#define STATUS_COMMITTING_LOG 2

#define ULONGSZ sizeof(uint32_t)
#define ULONGPT sizeof(uint32_t *)

struct versions {
	char			*string;
	uint32_t		val;
};

struct loserlist { 
	uint32_t		loser;
	uint32_t		size;
	struct loserlist	*next;
};      

extern struct versions known_versions[];

#define	ETYPE_ENUM	1
#define ETYPE_BATCH	2

#define NEXTINFO_MAGIC	0xEA

struct nis_nextinfo {
	unsigned char		cksum;
	unsigned char		etype;
	uint32_t		pos;
	uint32_t		nextcount;
/*
	uint32_t		winners[];
*/
};

struct nis_dbent {
	FILE			*fp;
	xff_header		hdr;
	FILE			*fp_log;
	xff_header		hdr_log;
	table_desc		*tdesc;
	char			*table_name;
	table_obj		*table;
	int			searchable;
	int			status;
	int			deletions;
	uint32_t		pos;
	uint32_t		*index_hashes;
	relation_array		**relarray;
};

typedef struct {
	void			*data;
	size_t			size;
} nis_datum;

extern u_int32_t hash __P(( const void *keyarg , register size_t len , int ));
extern db_status nis_db_parse_rels __P(( uint32_t, nis_attr *, struct nis_dbent * , uint32_t ** ));
extern db_status nis_db_process_addition __P(( struct nis_dbent * , entry_obj *, int, nis_attr * ));
extern void nis_db_unmerge_rel __P(( struct nis_dbent * , entry_obj * , int, nis_attr *, uint32_t ));
extern db_status validate_attrs __P(( struct nis_dbent * , uint32_t , nis_attr * ));
extern unsigned char __nis_db_cksum __P(( unsigned char *, unsigned int ));
extern char *encode_table_desc __P(( table_desc * , uint32_t * ));
extern table_desc *decode_table_desc __P(( char * , uint32_t ));
extern db_status cache_table __P(( char * , table_desc * ));
extern uint32_t cache_enum __P(( struct nis_dbent * , uint32_t ));
extern struct nis_dbent *find_table __P(( char * ));
extern struct nis_dbent *find_enum __P(( char * , uint32_t ));
extern db_status find_table_in_catalog __P (( char * ));
extern db_status flush_table __P(( char * ));
extern db_status flush_enum __P(( char * , uint32_t ));
extern db_status sync_table __P(( char * ));
extern db_status sync_all_tables __P(( void ));
extern db_status remove_table __P(( char * ));
extern db_status standby_table __P(( char * ));
extern db_status unstandby_table __P(( struct nis_dbent * ));
#ifdef notused
extern void flush_all __P(( void ));
#endif
extern db_status adjust_pos __P(( char * , struct loserlist * ));
extern db_status init_cache __P(( char * ));
extern db_status update_catalog __P(( char *, table_desc *, int ));
extern db_status rcm_get_index __P(( struct nis_dbent *, uint32_t, uint32_t, nis_datum *));
extern db_status rcm_put_index __P(( struct nis_dbent *, uint32_t, uint32_t, nis_datum *));
extern db_status rcm_delete_index __P(( struct nis_dbent *, uint32_t, uint32_t ));
extern db_status rcm_free __P(( struct nis_dbent * ));
extern db_status rcm_dump __P(( struct nis_dbent * ));
extern db_status rcm_load __P(( struct nis_dbent * ));
extern db_status rcm_compact __P(( struct loserlist * , struct nis_dbent * ));
extern db_status xff_create __P(( struct nis_dbent * , char * , int));
extern db_status xff_open __P(( struct nis_dbent * , char * ));
extern db_status xff_close __P(( struct nis_dbent * ));
extern db_status xff_compact_file __P(( struct nis_dbent * ));
extern db_status xff_get_entry __P(( struct nis_dbent * , entry_obj * , uint32_t ));
extern db_status xff_add_entry __P(( struct nis_dbent * , entry_obj * ));
extern db_status xff_delete_entry __P(( struct nis_dbent * , entry_obj * , uint32_t ));
extern db_status xff_first_entry __P(( struct nis_dbent * , entry_obj * ));
extern db_status xff_next_entry __P(( struct nis_dbent * , entry_obj * ));
extern db_status xff_add_log_entry __P(( struct nis_dbent *, entry_obj *, uint32_t, nis_attr * , xle_action ));
extern db_status xff_replay_log __P(( struct nis_dbent *, int ));
extern db_status xff_replay_dict_log __P(( struct nis_dbent *, int ));
extern db_status xff_commit_log __P (( struct nis_dbent * ));
extern db_status xff_commit_dict_log __P (( struct nis_dbent * ));

/* Should not be here. */
extern db_status put_dictionary __P(( char *, table_desc * ));
extern db_status del_dictionary __P(( char * ));
