/*
 * Copyright (c) 1996
 *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD$
 */

#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <limits.h>
#include <rpcsvc/nis.h>
#include <rpcsvc/nis_db.h>
#include "nis_db_private.h"

#ifndef lint
__RCSID("@(#) $FreeBSD$");
#endif

db_result *db_reset_next_entry(table_name, next_handle)
	char			*table_name;
	db_next_desc		*next_handle;
{
	db_result		*res;
	struct timeval		t_start, t_end;

	gettimeofday(&t_start, NULL);

	if ((res = calloc(1, sizeof(db_result))) == NULL)
		return(NULL);
	bzero((char *)res, sizeof(db_result));

	if (!next_handle->db_next_desc_len ||
		next_handle->db_next_desc_val == NULL) {
		res->status = DB_BADQUERY;
		goto bail;
	}

	next_handle->db_next_desc_len = 0;
	free(next_handle->db_next_desc_val);
	next_handle->db_next_desc_val = NULL;
	
bail:
	gettimeofday(&t_end, NULL);
	res->ticks += 
	    (t_end.tv_usec - t_start.tv_usec) +
	    ((t_end.tv_sec - t_start.tv_sec) * 1000000);

	return(res);
}
