/*
 * Copyright (c) 1996, 1997
 *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: nislib_error.c,v 1.1 1998/07/08 19:16:49 wpaul Exp $
 */

#include <sys/types.h>
#include <stdio.h>
#include <syslog.h>
#include <sys/cdefs.h>
#include <rpcsvc/nis.h>
#include <rpcsvc/nislib.h>
#include "nislib_private.h"

#ifndef lint
static const char rcsid[] = "@(#) $Id: nislib_error.c,v 1.1 1998/07/08 19:16:49 wpaul Exp $";
#endif

char *nis_sperrno(status)
	nis_error		status;
{
	char			*str;

	switch(status) {
	case NIS_SUCCESS:
		str = "Success";
		break;
	case NIS_S_SUCCESS:
		str = "Probable success";
		break;
	case NIS_NOTFOUND:
		str = "Not found";
		break;
	case NIS_S_NOTFOUND:
		str = "Probably not found";
		break;
	case NIS_CACHEEXPIRED:
		str = "Cache expired";
		break;
	case NIS_NAMEUNREACHABLE:
		str = "NIS+ servers unreachable";
		break;
	case NIS_UNKNOWNOBJ:
		str = "Unknown object";
		break;
	case NIS_TRYAGAIN:
		str = "Server busy, try again";
		break;
	case NIS_SYSTEMERROR:
		str = "Generic system error";
		break;
	case NIS_CHAINBROKEN:
		str = "First/Next chain broken";
		break;
	case NIS_PERMISSION:
		str = "Permission denied";
		break;
	case NIS_NOTOWNER:
		str = "Not owner";
		break;
	case NIS_NOT_ME:
		str = "Name not served by this server";
		break;
	case NIS_NOMEMORY:
		str = "Server out of memory";
		break;
	case NIS_NAMEEXISTS:
		str = "Object with same name exists";
		break;
	case NIS_NOTMASTER:
		str = "Not master server for this domain";
		break;
	case NIS_INVALIDOBJ:
		str = "Invalid Object for operation";
		break;
	case NIS_BADNAME:
		str = "Malformed Name, or illegal name";
		break;
	case NIS_NOCALLBACK:
		str = "Unable to create callback";
		break;
	case NIS_CBRESULTS:
		str = "Results sent to callback proc";
		break;
	case NIS_NOSUCHNAME:
		str = "Not Found, no such name";
		break;
	case NIS_NOTUNIQUE:
		str = "Name/entry isn't unique";
		break;
	case NIS_IBMODERROR:
		str = "Modification failed";
		break;
	case NIS_NOSUCHTABLE:
		str = "Database for table does not exist";
		break;
	case NIS_TYPEMISMATCH:
		str = "Entry/table type mismatch";
		break;
	case NIS_LINKNAMEERROR:
		str = "Link Points to illegal name";
		break;
	case NIS_PARTIAL:
		str = "Partial Success";
		break;
	case NIS_TOOMANYATTRS:
		str = "Too Many Attributes";
		break;
	case NIS_RPCERROR:
		str = "Error in RPC subsystem";
		break;
	case NIS_BADATTRIBUTE:
		str = "Missing or malformed attribute";
		break;
	case NIS_NOTSEARCHABLE:
		str = "Named object is not searchable";
		break;
	case NIS_CBERROR:
		str = "Error while talking to callback proc";
		break;
	case NIS_FOREIGNNS:
		str = "Non NIS+ namespace encountered";
		break;
	case NIS_BADOBJECT:
		str = "Illegal object type for operation";
		break;
	case NIS_NOTSAMEOBJ:
		str = "Passed object is not the same object on server";
		break;
	case NIS_MODFAIL:
		str = "Modify operation failed";
		break;
	case NIS_BADREQUEST:
		str = "Query illegal for named table";
		break;
	case NIS_NOTEMPTY:
		str = "Attempt to remove non-empty table";
		break;
	case NIS_COLDSTART_ERR:
		str = "Error in accessing NIS+ cold start file, \
is NIS+ installed";
		break;
	case NIS_RESYNC:
		str = "Full resync required for directory";
		break;
	case NIS_FAIL:
		str = "NIS+ operation failed";
		break;
	case NIS_UNAVAIL:
		str = "NIS+ service is unavailable or not installed";
		break;
	case NIS_RES2BIG:
		str = "Response too big for datagram";
		break;
	case NIS_SRVAUTH:
		str = "Unable to authenticate NIS+ server";
		break;
	case NIS_CLNTAUTH:
		str = "Unable to authenticate NIS+ client";
		break;
	case NIS_NOFILESPACE:
		str = "No file space on server";
		break;
	case NIS_NOPROC:
		str = "Unable to create process on server";
		break;
	case NIS_DUMPLATER:
		str = "Master server is busy, full dump rescheduled";
		break;
	default:
		str = "unknown error";
		break;
	}

	return(str);
}

void nis_perror(status, label)
	nis_error		status;
	char			*label;
{
	fprintf(stderr, "%s: %s\n", label, nis_sperrno(status));

	return;
}

void nis_lerror(status, label)
	nis_error		status;
	char			*label;
{
	syslog(LOG_ERR, "%s: %s", label, nis_sperrno(status));

	return;
}

char *nis_sperror_r(status, label, buf)
	nis_error		status;
	char			*label;
	char			*buf;
{
	sprintf(buf, "%s: %s", label, nis_sperrno(status));

	return(buf);
}

char *nis_sperror(status, label)
	nis_error		status;
	char			*label;
{
	static char		errbuf[128];

	snprintf(errbuf, 128, "%s: %s", label, nis_sperrno(status));

	return(errbuf);
}

