/*
 * Copyright (c) 1996, 1997, 1998
 *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	$Id: nislib_local_names.c,v 1.1 1998/07/08 19:16:49 wpaul Exp wpaul $
 */

#include <sys/types.h>
#include <sys/cdefs.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <rpcsvc/nis.h>
#include <rpcsvc/yp_prot.h>
#include <rpcsvc/ypclnt.h>
#include "nislib_private.h"

#ifndef lint
static const char rcsid[] =
	"@(#) $Id: nislib_local_names.c,v 1.1 1998/07/08 19:16:49 wpaul Exp wpaul $";
#endif

nis_name nis_local_domain()
{
	nis_name		domain = NULL;
	static char		lbuf[NIS_MAXNAMELEN];

	yp_get_default_domain((char **)&domain);
	if (domain == NULL || strlen (domain) == 0)
		domain = "blah";
	snprintf(lbuf, sizeof(lbuf), "%s.", domain);

	return ((nis_name)&lbuf);
}

nis_name nis_local_directory()
{
	return(nis_local_domain());
}

nis_name nis_local_host()
{
	static char		hostname[NIS_MAXNAMELEN + 2];

	gethostname(hostname, NIS_MAXNAMELEN);

	if (strchr(hostname, '.') != NULL)
		*strchr(hostname, '.') = '\0';

	strcat(hostname, ".");
	strcat(hostname, nis_local_domain());

	return((char *)&hostname);
}

nis_name nis_local_group()
{
	char			*grp = getenv("NIS_GROUP");
	static char		buf[NIS_MAXNAMELEN];

	if (grp == NULL)
		return("");

	if (grp[strlen(grp) - 1] != '.')
		snprintf(buf, sizeof(buf), "%s.%s", grp, nis_local_directory());
	else
		snprintf(buf, sizeof(buf), "%s", grp);

	return((nis_name)&buf);
}

nis_name nis_local_principal()
{
	char			qbuf1[NIS_MAXNAMELEN];
	char			qbuf2[NIS_MAXNAMELEN];
	nis_result		*res;
	static char		princ[NIS_MAXNAMELEN];
	struct nis_attr		attr[2] = { { "auth_type", { 6, "LOCAL" } },
					    { "auth_name", { 0, NULL } } };

	strcpy(princ, "nobody");

	if (!geteuid())
		return(nis_local_host());


	snprintf(qbuf1, sizeof(qbuf1), "%lu", geteuid());
	snprintf(qbuf2, sizeof(qbuf2), "cred.org_dir.%s",
					nis_local_directory());
	attr[1].zattr_val.zattr_val_val = qbuf1;
	attr[1].zattr_val.zattr_val_len = strlen(qbuf1) + 1;

	res = __nis_search(qbuf2, SEARCHOP_LIST, 0, attr, 2, NULL, NULL);

	if (res == NULL)
		return(princ);

	if (res->status != NIS_SUCCESS ||
	    __type_of(NIS_RES_OBJECT(res)) != ENTRY_OBJ ||
	    strcmp(NIS_RES_OBJECT(res)->EN_data.en_type, "cred_tbl")) {
		nis_freeresult(res);
		return(princ);
	}

	snprintf(princ, sizeof(princ), "%s",
		ENTRY_VAL(NIS_RES_OBJECT(res), 0));

	nis_freeresult(res);

	return(princ);
}
