/*
 * Copyright (c) 1996, 1997
 *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: nislib_ping.c,v 1.1 1998/07/08 19:16:49 wpaul Exp wpaul $
 */

/*
 * What follows is a special version of clntudp_call() that has been
 * hacked to send requests and receive replies asynchronously. Similar
 * magic is used inside rpc.nisd(8) for the special non-blocking,
 * non-fork()ing, non-threading callback support.
 *
 * What's special about this version of clntudp_call() is that it
 * doesn't check the RPC transaction ID (XID) in the replies that
 * it receives. What __nis_ping() does is to transmit a series of
 * of requests to the NULLPROC to all the endpoints it can find,
 * using a different XID for each request. It then grabs the first
 * reply that it gets and checks the XID; this XID can then be used
 * to go through the list of endpoints to find the one that was
 * used to send the request. This is slightly more effective than
 * just checking the IP address in the response packet. Unfortunately.
 * there's no simple way to achieve this with the stock clntudp_call(),
 * so we need our own slightly modified version here.
 */

/*
 * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify Sun RPC without charge, but are not authorized
 * to license or distribute it to anyone else except as part of a product or
 * program developed by the user.
 *
 * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 *
 * Sun RPC is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 *
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
 * OR ANY PART THEREOF.
 *
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even if
 * Sun has been advised of the possibility of such damages.
 *
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */

#ifndef lint
/*static char *sccsid = "from: @(#)clnt_udp.c 1.39 87/08/11 Copyr 1984 Sun Micro";*/
/*static char *sccsid = "from: @(#)clnt_udp.c	2.2 88/08/01 4.0 RPCSRC";*/
static const char rcsid[] =
	"@(#) $Id: nislib_ping.c,v 1.1 1998/07/08 19:16:49 wpaul Exp wpaul $";
#endif

/*
 * clnt_udp.c, Implements a UDP/IP based, client side RPC.
 *
 * Copyright (C) 1984, Sun Microsystems, Inc.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <rpc/rpc.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/event.h>
#include <sys/cdefs.h>
#include <netdb.h>
#include <errno.h>
#include <rpc/pmap_clnt.h>
#include <rpc/pmap_prot.h>
#include <rpcsvc/nis.h>
#include <rpcsvc/nislib.h>
#include "nislib_private.h"

/*
 * Compatibility with other systems that don't have these macros.
 */
#ifndef timeradd
#ifndef KERNEL		/* use timevaladd/timevalsub in kernel */
/* NetBSD/OpenBSD compatable interfaces */
#define timeradd(tvp, uvp, vvp)						\
	do {								\
		(vvp)->tv_sec = (tvp)->tv_sec + (uvp)->tv_sec;		\
		(vvp)->tv_usec = (tvp)->tv_usec + (uvp)->tv_usec;	\
		if ((vvp)->tv_usec >= 1000000) {			\
			(vvp)->tv_sec++;				\
			(vvp)->tv_usec -= 1000000;			\
		}							\
	} while (0)
#define timersub(tvp, uvp, vvp)						\
	do {								\
		(vvp)->tv_sec = (tvp)->tv_sec - (uvp)->tv_sec;		\
		(vvp)->tv_usec = (tvp)->tv_usec - (uvp)->tv_usec;	\
		if ((vvp)->tv_usec < 0) {				\
			(vvp)->tv_sec--;				\
			(vvp)->tv_usec += 1000000;			\
		}							\
	} while (0)
#endif
#endif

/*
 * Private data kept per client handle
 */
#define       MCALL_MSG_SIZE 24

struct cu_data {
        int                     cu_fd;          /* connections fd */
        bool_t                  cu_closeit;     /* opened by library */
        struct sockaddr_storage cu_raddr;       /* remote address */
        int                     cu_rlen;
        struct timeval          cu_wait;        /* retransmit interval */
        struct timeval          cu_total;       /* total time for the call */
        struct rpc_err          cu_error;
        XDR                     cu_outxdrs;
        u_int                   cu_xdrpos;
        u_int                   cu_sendsz;      /* send size */
        char                    cu_outhdr[MCALL_MSG_SIZE];
        char                    *cu_outbuf;
        u_int                   cu_recvsz;      /* recv size */
        int                     cu_async;
        int                     cu_connect;     /* Use connect(). */
        int                     cu_connected;   /* Have done connect(). */
        struct kevent           cu_kin;
        int                     cu_kq;
        char                    cu_inbuf[1];
};

static enum clnt_stat
clntudp_a_call(cl, proc, xargs, argsp, xresults, resultsp, utimeout)
	register CLIENT	*cl;		/* client handle */
	rpcproc_t	proc;		/* procedure number */
	xdrproc_t	xargs;		/* xdr routine for args */
	caddr_t		argsp;		/* pointer to args */
	xdrproc_t	xresults;	/* xdr routine for results */
	caddr_t		resultsp;	/* pointer to results */
	struct timeval	utimeout;	/* seconds to wait before giving up */
{
	register struct cu_data *cu = (struct cu_data *)cl->cl_private;
	register XDR *xdrs;
	register int outlen = 0;
	register int inlen;
	socklen_t fromlen;
	fd_set *fds, readfds;
	struct sockaddr_in from;
	struct rpc_msg reply_msg;
	XDR reply_xdrs;
	struct timeval time_waited, start, after, tmp1, tmp2, tv;
	bool_t ok;
	int nrefreshes = 2;	/* number of times to refresh cred */
	struct timeval timeout;
	uint32_t xid;

	if (cu->cu_total.tv_usec == -1)
		timeout = utimeout;     /* use supplied timeout */
	else
		timeout = cu->cu_total; /* use default timeout */

	if (cu->cu_fd + 1 > FD_SETSIZE) {
		int bytes = _howmany(cu->cu_fd + 1, NFDBITS) * sizeof(fd_mask);
		fds = (fd_set *)malloc(bytes);
		if (fds == NULL)
			return (cu->cu_error.re_status = RPC_CANTSEND);
		memset(fds, 0, bytes);
	} else {
		fds = &readfds;
		FD_ZERO(fds);
	}

	timerclear(&time_waited);

call_again:
	xdrs = &(cu->cu_outxdrs);
	if (xargs == NULL)
		goto get_reply;
	xdrs->x_op = XDR_ENCODE;
	XDR_SETPOS(xdrs, 0);
	/*
	 * the transaction is the first thing in the out buffer
	 */
	xid = ntohl(*(u_int32_t *)(void *)(cu->cu_outhdr));
	xid++;
 	*(u_int32_t *)(void *)(cu->cu_outhdr) = htonl(xid);
	if ((! XDR_PUTBYTES(xdrs, cu->cu_outhdr, cu->cu_xdrpos)) ||
	    (! XDR_PUTINT32(xdrs, (int32_t *)&proc)) ||
	    (! AUTH_MARSHALL(cl->cl_auth, xdrs)) ||
	    (! (*xargs)(xdrs, argsp))) {
		if (fds != &readfds)
			free(fds);
		return (cu->cu_error.re_status = RPC_CANTENCODEARGS);
	}
	outlen = (int)XDR_GETPOS(xdrs);

send_again:
	if (sendto(cu->cu_fd, cu->cu_outbuf, outlen, 0,
	    (struct sockaddr *)&(cu->cu_raddr), cu->cu_rlen) != outlen) {
		cu->cu_error.re_errno = errno;
		if (fds != &readfds)
			free(fds);
		return (cu->cu_error.re_status = RPC_CANTSEND);
	}

	/*
	 * Hack to provide rpc-based message passing
	 */
	if (!timerisset(&timeout)) {
		if (fds != &readfds)
			free(fds);
		return (cu->cu_error.re_status = RPC_TIMEDOUT);
	}

get_reply:

	/*
	 * sub-optimal code appears here because we have
	 * some clock time to spare while the packets are in flight.
	 * (We assume that this is actually only executed once.)
	 */
	reply_msg.acpted_rply.ar_verf = _null_auth;
	reply_msg.acpted_rply.ar_results.where = resultsp;
	reply_msg.acpted_rply.ar_results.proc = xresults;

	gettimeofday(&start, NULL);
	for (;;) {
		/* XXX we know the other bits are still clear */
		FD_SET(cu->cu_fd, fds);
		tv = cu->cu_wait;
		switch (select(cu->cu_fd+1, fds, NULL, NULL, &tv)) {

		case 0:
			timeradd(&time_waited, &cu->cu_wait, &tmp1);
			time_waited = tmp1;
			if (timercmp(&time_waited, &timeout, <))
				goto send_again;
			if (fds != &readfds)
				free(fds);
			return (cu->cu_error.re_status = RPC_TIMEDOUT);

		case -1:
			if (errno == EINTR) {
				gettimeofday(&after, NULL);
				timersub(&after, &start, &tmp1);
				timeradd(&time_waited, &tmp1, &tmp2);
				time_waited = tmp2;
				if (timercmp(&time_waited, &timeout, <))
					continue;
				if (fds != &readfds)
					free(fds);
				return (cu->cu_error.re_status = RPC_TIMEDOUT);
			}
			cu->cu_error.re_errno = errno;
			if (fds != &readfds)
				free(fds);
			return (cu->cu_error.re_status = RPC_CANTRECV);
		}

		do {
			fromlen = sizeof(struct sockaddr);
			inlen = recvfrom(cu->cu_fd, cu->cu_inbuf,
				(int) cu->cu_recvsz, 0,
				(struct sockaddr *)&from, &fromlen);
		} while (inlen < 0 && errno == EINTR);
		if (inlen < 0) {
			if (errno == EWOULDBLOCK)
				continue;
			cu->cu_error.re_errno = errno;
			if (fds != &readfds)
				free(fds);
			return (cu->cu_error.re_status = RPC_CANTRECV);
		}
		if (inlen < sizeof(u_int32_t))
			continue;

		/*
		 * XXX Do not throw out replies based on XID
		 * mismatches. Since we don't know which server
		 * will reply first, we can't know which XID is
		 * the 'correct' one.
		 */
#ifdef CHECK_XID
		/* see if reply transaction id matches sent id */
		if (*((u_int32_t *)(cu->cu_inbuf)) != *((u_int32_t *)(cu->cu_outbuf)))
			continue;
#endif
		/* we now assume we have the proper reply */
		break;
	}

	/*
	 * now decode and validate the response
	 */
	xdrmem_create(&reply_xdrs, cu->cu_inbuf, (u_int)inlen, XDR_DECODE);
	ok = xdr_replymsg(&reply_xdrs, &reply_msg);
	/* XDR_DESTROY(&reply_xdrs);  save a few cycles on noop destroy */
	if (ok) {
		_seterr_reply(&reply_msg, &(cu->cu_error));
		if (cu->cu_error.re_status == RPC_SUCCESS) {
#ifdef __sgi
			if (! AUTH_VALIDATE(cl->cl_auth,
				reply_msg.acpted_rply.ar_verf)) {
#else
			if (! AUTH_VALIDATE(cl->cl_auth,
				&reply_msg.acpted_rply.ar_verf)) {
#endif
				cu->cu_error.re_status = RPC_AUTHERROR;
				cu->cu_error.re_why = AUTH_INVALIDRESP;
			}
			if (reply_msg.acpted_rply.ar_verf.oa_base != NULL) {
				xdrs->x_op = XDR_FREE;
				(void)xdr_opaque_auth(xdrs,
				    &(reply_msg.acpted_rply.ar_verf));
			}
		}  /* end successful completion */
		else {
			/* maybe our credentials need to be refreshed ... */
			if (nrefreshes > 0 && AUTH_REFRESH(cl->cl_auth, &reply_msg)) {
				nrefreshes--;
				goto call_again;
			}
		}  /* end of unsuccessful completion */
	}  /* end of valid reply message */
	else {
		int op = reply_xdrs.x_op;
		reply_xdrs.x_op = XDR_FREE;
		xdr_replymsg(&reply_xdrs, &reply_msg);
		reply_xdrs.x_op = op;
		cu->cu_error.re_status = RPC_CANTDECODERES;
	}
	if (fds != &readfds)
		free(fds);
	return (cu->cu_error.re_status);
}


/*
 * Transmit to NULLPROC, return immediately.
 */
static void *
nis_null_3_send(void *argp, CLIENT *clnt)
{
	static char clnt_res;
	struct timeval TIMEOUT = { 0, 0 };

	memset((char *)&clnt_res, 0, sizeof (clnt_res));
	if (clnt_call(clnt, NULLPROC,
		(xdrproc_t) xdr_void, (caddr_t) argp,
		(xdrproc_t) xdr_void, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return ((void *)&clnt_res);
}

/*
 * Receive request from NULLPROC asynchronously.
 */
static void *
nis_null_3_recv(void *argp, CLIENT *clnt)
{
	static char clnt_res;
	struct timeval TIMEOUT = { 0, 0 };

	memset((char *)&clnt_res, 0, sizeof (clnt_res));
	if (clnt_call(clnt, NULLPROC,
		(xdrproc_t) NULL, (caddr_t) argp,
		(xdrproc_t) xdr_void, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return ((void *)&clnt_res);
}

/*
 * pmap_getport.c
 * Client interface to pmap rpc service.
 *
 * Copyright (C) 1984, Sun Microsystems, Inc.
 */

#ifdef foo
static struct timeval timeout = { 1, 0 };
#endif
static struct timeval tottimeout = { 1, 0 };

/*
 * Find the mapped port for program,version.
 * Calls the pmap service remotely to do the lookup.
 * Returns 0 if no map exists.
 */
static u_short
__pmap_getport(address, program, version, protocol)
	struct sockaddr_in *address;
	u_long program;
	u_long version;
	u_int protocol;
{
	u_short port = 0;
	int sock = -1;
	register CLIENT *client;
	struct pmap parms;

	address->sin_port = htons(PMAPPORT);
#ifdef foo
	client = clntudp_bufcreate(address, PMAPPROG,
#endif
	client = clnttcp_create(address, PMAPPROG,
	    PMAPVERS, /*timeout,*/ &sock, RPCSMALLMSGSIZE, RPCSMALLMSGSIZE);
	if (client != (CLIENT *)NULL) {
		parms.pm_prog = program;
		parms.pm_vers = version;
		parms.pm_prot = protocol;
		parms.pm_port = 0;  /* not needed or used */
		if (CLNT_CALL(client, PMAPPROC_GETPORT, (xdrproc_t)xdr_pmap, &parms,
		    (xdrproc_t)xdr_u_short, &port, tottimeout) != RPC_SUCCESS){
			rpc_createerr.cf_stat = RPC_PMAPFAILURE;
			clnt_geterr(client, &rpc_createerr.cf_error);
		} else if (port == 0) {
			rpc_createerr.cf_stat = RPC_PROGNOTREGISTERED;
		}
		auth_destroy(client->cl_auth);
		CLNT_DESTROY(client);
	}
	if (sock != -1)
		(void)close(sock);
	address->sin_port = 0;
	return (port);
}

/*
 * "We have the machine that goes 'ping!'" -- Monty Python
 *
 * This function blasts packets at the NULLPROC procedure of the
 * NIS+ servers listed in the supplied directory_obj structure.
 * Whoever replies the fastest becomes our chosen server.
 *
 * Note: THIS IS NOT A BROADCAST OPERATION! We could use clnt_broadcast()
 * for this, but that has the following problems:
 * - We only get the address of the machine that replied in the
 *   'eachresult' callback, and on multi-homed machines this can
 *   lead to confusion.
 * - clnt_broadcast() only transmits to local networks, whereas with
 *   NIS+ you can have a perfectly good server located anywhere on or
 *   off the local network.
 * - clnt_broadcast() blocks for an arbitrary amount of time which the
 *   caller can't control -- we want to avoid that.
 *
 * Also note that this has nothing to do with the NIS_PING procedure used
 * for replica updates.
 */

struct ping_req {
	struct sockaddr_in	sin;
	nis_server		*srv;
	endpoint		*ep;
	uint32_t		srv_idx;
	uint32_t		ep_idx;
	uint32_t		xid;
};

nis_error __nis_ping(bptr)
	struct _nis_binding	*bptr;
{
	struct timeval		tv = { 5 , 0 };
	struct ping_req		**reqs;
	uint32_t		i, j;
	endpoint		*ep;
	nis_server		*srv = NULL;
	directory_obj		*dir = NULL;
	struct sockaddr_in	xsin, sin;
	uint32_t		xid_seed, xid_lookup;
	int			sock, dontblock = 1;
	CLIENT			*clnt;
	void			*foo = NULL;
	struct cu_data		*cu;
	enum clnt_stat		(*oldfunc)();
	int			validsrvs = 0;
	int			checklen = sizeof(struct sockaddr_in);
	struct sockaddr_in	local;
	struct timeval		seed;
	struct manual_binding	*m;

	/* Set up handles. */
	reqs = calloc(1, sizeof(struct ping_req *));
	gettimeofday(&seed, NULL);
	xid_seed = (seed.tv_sec + seed.tv_usec) ^ getpid();

	m = bptr->nb_binding_manual;
	dir = m->mb_dir;

	if (dir == NULL) {
		srv = m->mb_srv;
		for (j = 0; j < srv->ep.ep_len; j++) {
			if (srv->name == NULL)
				return(NIS_NAMEUNREACHABLE);
			ep = &srv->ep.ep_val[j];
			if (bptr->nb_flags & ZMH_DG) {
				if (strcmp(ep->proto, "udp"))
					continue;
			} else {
				if (strcmp(ep->proto, "tcp"))
				continue;
			}
			if (__nis_uaddr_to_sockaddr(ep, &sin)
						 != NIS_SUCCESS)
				continue;
			sin.sin_port = htons(__pmap_getport(&sin,
				NIS_PROG, NIS_VERSION, IPPROTO_UDP));
			if (!sin.sin_port)
				continue;
			reqs = realloc(reqs, sizeof(struct ping_req *) *
					(validsrvs + 1));
			reqs[validsrvs] = calloc(1, sizeof(struct ping_req));
			bcopy((char *)&sin, (char *)&reqs[validsrvs]->sin,
						sizeof(sin));
			bcopy((char *)&sin, (char *)&xsin, sizeof(sin));
			reqs[validsrvs]->xid = xid_seed;
			reqs[validsrvs]->srv = srv;
			reqs[validsrvs]->ep = ep;
			reqs[validsrvs]->srv_idx = -1;
			reqs[validsrvs]->ep_idx = j;
			xid_seed++;
			validsrvs++;
		}
	} else {
		for (i = 0; i < dir->do_servers.do_servers_len; i++) {
			if (dir->do_servers.do_servers_val[i].name == NULL)
				continue;
			srv = &dir->do_servers.do_servers_val[i];
			for (j = 0; j < srv->ep.ep_len; j++) {
				ep = &srv->ep.ep_val[j];
				if (bptr->nb_flags & ZMH_DG) {
					if (strcmp(ep->proto, "udp"))
						continue;
				} else {
					if (strcmp(ep->proto, "tcp"))
					continue;
				}
				if (__nis_uaddr_to_sockaddr(ep, &sin)
							 != NIS_SUCCESS)
					continue;
				sin.sin_port = htons(__pmap_getport(&sin,
					NIS_PROG, NIS_VERSION, IPPROTO_UDP));
				if (!sin.sin_port)
					continue;
				reqs = realloc(reqs, sizeof(struct ping_req *) *
						(validsrvs + 1));
				reqs[validsrvs] =
					 calloc(1, sizeof(struct ping_req));
				bcopy((char *)&sin,
					 (char *)&reqs[validsrvs]->sin,
							sizeof(sin));
				bcopy((char *)&sin, (char *)&xsin, sizeof(sin));
				reqs[validsrvs]->xid = xid_seed;
				reqs[validsrvs]->srv = srv;
				reqs[validsrvs]->ep = ep;
				reqs[validsrvs]->srv_idx = i;
				reqs[validsrvs]->ep_idx = j;
				xid_seed++;
				validsrvs++;
			}
		}
	}

	/* Make sure at least one server was assigned */
	if (!validsrvs) {
		free(reqs);
		return(NIS_FAIL);
	}

	/* Create RPC handle */
	sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	clnt = clntudp_create(&xsin, NIS_PROG, NIS_VERSION, tv, &sock);
	if (clnt == NULL) {
		close(sock);
		for (i = 0; i < validsrvs; i++)
			if (reqs[i] != NULL)
				free(reqs[i]);
		free(reqs);
		return(NIS_FAIL);
	}
	auth_destroy(clnt->cl_auth);
	clnt->cl_auth = authunix_create_default();
	cu = (struct cu_data *)clnt->cl_private;
	tv.tv_sec = 0;
	clnt_control(clnt, CLSET_TIMEOUT, &tv);
	ioctl(sock, FIONBIO, &dontblock);
	bzero((char *)&local, sizeof(local));
	bind(sock, (struct sockaddr *)&local, checklen);
	oldfunc = clnt->cl_ops->cl_call;
	clnt->cl_ops->cl_call = clntudp_a_call;

	/* Transmit */
	for (i = 0; i < validsrvs; i++) {
		if (reqs[i] != NULL) {
			/* subtract one; clntudp_call() will increment */
			*((u_int32_t *)(cu->cu_outhdr)) =
				htonl(reqs[i]->xid - 1);
			bcopy((char *)&reqs[i]->sin, (char *)&cu->cu_raddr,
				sizeof(struct sockaddr_in));
			nis_null_3_send(foo, clnt);
		}
	}

	/* Receive reply */
	nis_null_3_recv(foo, clnt);

	/* Got a winner -- look him up. */
	xid_lookup = *((u_int32_t *)(cu->cu_inbuf));
	xid_lookup = ntohl(xid_lookup);
	for (i = 0; i < validsrvs; i++) {
		if (reqs[i] != NULL && reqs[i]->xid == xid_lookup) {
			m->mb_srv = reqs[i]->srv;
			m->mb_srv_idx = reqs[i]->srv_idx;
			m->mb_ep_idx = reqs[i]->ep_idx;
			bcopy((char *)&reqs[i]->sin, (char *)&bptr->nb_srv_addr,
					sizeof(struct sockaddr_in));
		}
	}

	/* Shut everything down */
	clnt->cl_ops->cl_call = oldfunc;
	auth_destroy(clnt->cl_auth);
	clnt_destroy(clnt);
	close(sock);

	for (i = 0; i < validsrvs; i++)
		if (reqs[i] != NULL)
			free(reqs[i]);
	free(reqs);

	return(NIS_SUCCESS);
}
