/*
 * Copyright (c) 1996, 1997
 *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	$Id: nisdefaults.c,v 1.7 1997/09/05 16:52:58 wpaul Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/cdefs.h>
#include <rpcsvc/nis.h>
#include <rpcsvc/nislib.h>
#include "nis_defaults.h"

#ifndef lint
static const char copyright[] = "@(#) Copyright (c) 1996, 1997\
	Bill Paul. All rights reserved.";
static const char rcsid[] = "@(#) $Id: nisdefaults.c,v 1.7 1997/09/05 16:52:58 wpaul Exp $";
#endif

static char *ttltostr(ttl)
	unsigned long		ttl;
{
	int			hours = 0,
				mins = 0,
				secs = 0;

	static char		str[20];

	hours = ttl / 3600;
	mins = (ttl - (hours * 3600)) / 60;
	secs = ttl - (hours * 3600) - (mins * 60);

	snprintf(str, sizeof(str), "%02d:%02d:%02d", hours, mins, secs);

	return(str);
}

void usage(pname)
	char			*pname;
{
	fprintf(stderr, "usage: %s [ -adghprstv ]\n", pname);
	fprintf(stderr, " p = default principal name.\n");
	fprintf(stderr, " d = default domain name.\n");
	fprintf(stderr, " h = default host name.\n");
	fprintf(stderr, " g = default group name.\n");
	fprintf(stderr, " r = default access rights.\n");
	fprintf(stderr, " t = default time to live.\n");
	fprintf(stderr, " s = default search path.\n");
	fprintf(stderr, " a = all of the above(default).\n");
	fprintf(stderr, " v = verbose.\n");

	exit(1);
}

int main(argc, argv)
	int			argc;
	char			*argv[];
{
	int			ch;
	int			dom = 0, grp = 0;
	int			host = 0, princ = 0, acc = 0;
	int			path = 0, ttl = 0, verbose = 0;
	int			uops = 0;
	nis_name		*n, *ntmp;

	while((ch = getopt(argc, argv, "adghprstv")) != -1) {
		switch(ch) {
		case 'a':
			uops++; dom = 1; grp = 1;
			host = 1; princ = 1; acc = 1;
			path = 1; ttl = 1;
			verbose = 0;
			break;
		case 'd':
			uops++; dom++;
			break;
		case 'g':
			uops++; grp++;
			break;
		case 'h':
			uops++; host++;
			break;
		case 'p':
			uops++; princ++;
			break;
		case 'r':
			uops++; acc++;
			break;
		case 's':
			uops++; path++;
			break;
		case 't':
			uops++; ttl++;
			break;
		case 'v':
			verbose = 1;
			break;
		default:
			usage(argv[0]);
			break;
		}
	}

	if (!uops) {
		verbose = 1; dom = 1; grp = 1;
		host = 1; princ = 1; acc = 1;
		path = 1; ttl = 1;
	}

	if (princ) {
		if (verbose)
			printf("Principal Name : ");
		printf("%s", nis_default_owner());
		if (!key_secretkey_is_set())
			printf(" (not authenticated)");
		printf("\n");
	}

	if (dom) {
		if (verbose)
			printf("Domain Name    : ");
		printf("%s\n", nis_local_directory());
	}

	if (host) {
		if (verbose)
			printf("Host Name      : ");
		printf("%s\n", nis_local_host());
	}

	if (grp) {
		if (verbose)
			printf("Group Name     : ");
		printf("%s\n", nis_default_group());
	}

	if (acc) {
		if (verbose)
			printf("Access Rights  : ");
		nis_print_rights(nis_default_rights());
		printf("\n");
	}

	if (ttl) {
		if (verbose) {
			printf("Time to live   : ");
			printf("%s\n", ttltostr(nis_default_ttl()));
		} else
			printf("%lu\n", nis_default_ttl());
	}

	if (path) {
		int			skip = 1;
		if (verbose)
			printf("Search Path    : ");
		n = ntmp = nis_getnames("foo");
		if (n == NULL)
			printf("**NONE**");
		else {
			while(*ntmp != NULL) {
				if (verbose & !skip)
					printf("                 ");
				skip = 0;
				printf("%s\n", nis_domain_of(*ntmp));
				ntmp++;
			}
			nis_freenames(n);
		}
		printf("\n");
	}

	exit(0);
}

