/*
 * Copyright (c) 1996, 1997
 *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	$Id: nisln.c,v 1.4 1998/06/02 22:42:20 wpaul Exp $
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <err.h>
#include <sys/cdefs.h>
#include <rpcsvc/nis.h>
#include <rpcsvc/nislib.h>
#include "nis_defaults.h"

#ifndef lint
static const char copyright[] = "@(#) Copyright (c) 1996, 1997\
	Bill Paul. All rights reserved.";
static const char rcsid[] =
	"@(#) $Id: nisln.c,v 1.4 1998/06/02 22:42:20 wpaul Exp $";
#endif

/*
 * Abuse this hidden routine in the NIS+ client library.
 */
extern nis_error __nis_parse_request __P(( nis_name, nis_attr **,
					   unsigned long *, nis_name * ));

void usage(pname)
	char			*pname;
{
	fprintf(stderr, "%s: [-D defaults] [-L] name linkname\n", pname);
	exit(1);
}

int main(argc, argv)
	int			argc;
	char			*argv[];
{
	int			ch;
	nis_result		*res;
	nis_object		obj;
	link_obj		*l;
	unsigned long		flags = MASTER_ONLY;
	char			*defs = NULL, *pname = argv[0];
	nis_name		tname;
	nis_attr		*attr;
	unsigned long		numattrs;

	while((ch = getopt(argc, argv, "LD:")) != -1) {
		switch(ch) {
		case 'L':
			flags |= FOLLOW_LINKS;
			break;
		case 'D':
			defs = optarg;
			break;
		default:
			usage(argv[0]);
			break;
		}
	}

	argc -= optind;
	argv += optind;

	if (argc < 2)
		usage(pname);

	bzero((char *)&obj, sizeof(obj));
	bzero((char *)&l, sizeof(l));

	if (argv[0][strlen(argv[0]) - 1] != '.')
		errx(1, "object name must be fully qualified.");

	if (argv[1][strlen(argv[1]) - 1] != '.')
		errx(1, "link name must be fully qualified.");

	res = nis_lookup(nis_domain_of(argv[1]), 0);

	if (res == NULL)
		errx(1, "out of memory doing nis_lookup");

	if (res->status != NIS_SUCCESS) {
		warnx("bad link domain '%s': %s", nis_domain_of(argv[1]),
					 nis_sperrno(res->status));
		nis_freeresult(res);
		exit(1);
	}

	nis_freeresult(res);

	if (strchr(argv[0], '[') != NULL)
		res = nis_list(argv[0], flags, NULL, NULL);
	else
		res = nis_lookup(argv[0], flags);

	if (res == NULL)
		errx(1, "out of memory doing nis_lookup");

	if (res->status != NIS_SUCCESS) {
		warnx("%s: %s", argv[0], nis_sperrno(res->status));
		nis_freeresult(res);
		exit(1);
	}

	if (__type_of(NIS_RES_OBJECT(res)) == DIRECTORY_OBJ) {
		warnx("%s is a directory!", argv[0]);
		nis_freeresult(res);
		exit(1);
	}

	nis_init_defs(defs);

	obj.zo_name = strdup(nis_leaf_of(argv[1]));
	obj.zo_domain = strdup(nis_domain_of(argv[1]));
	obj.zo_owner = strdup(nis_default_owner());
	obj.zo_group = strdup(nis_default_group());
	obj.zo_access = nis_default_rights();
	obj.zo_ttl = nis_default_ttl();
	__type_of(&obj) = LINK_OBJ;
	l = &obj.LI_data;

	l->li_rtype = __type_of(NIS_RES_OBJECT(res));

	if (strchr(argv[0], '[')) {
		if (__nis_parse_request(argv[0], &attr,
				 &numattrs, &tname) != NIS_SUCCESS)
			errx(1, "badly formed indexed name: %s", argv[1]);
		l->li_attrs.li_attrs_len = numattrs;
		l->li_attrs.li_attrs_val = attr;
		l->li_name = tname;
	} else {
		l->li_name = argv[0];
	}

	res = nis_add(argv[1], &obj);

	if (res == NULL)
		errx(1, "out of memory doing nis_add");

	if (res->status != NIS_SUCCESS) {
		warnx("failed to add link: %s", nis_sperrno(res->status));
		nis_freeresult(res);
		exit(1);
	}

	nis_freeresult(res);

	exit(0);
}
