/*
 * Copyright (c) 1996, 1997, 1998
 *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	$Id: nisrm.c,v 1.1 2000/01/24 00:09:22 root Exp root $
 */

#include <sys/types.h>
#include <sys/cdefs.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <err.h>
#include <rpcsvc/nis.h>
#include <rpcsvc/nislib.h>

#ifndef lint
static const char copyright[] = "@(#) Copyright (c) 1996, 1997\
	Bill Paul. All rights reserved.";
static const char rcsid[] =
	"@(#) $Id: nisrm.c,v 1.1 2000/01/24 00:09:22 root Exp root $";
#endif

static void usage(pname)
	char			*pname;
{
	fprintf(stderr, "%s: [-if] name ...\n", pname);
	exit(1);
}

static void try_force(name, o)
	nis_name		name;
	nis_object		*o;
{
	nis_result		*res;

	/*
	 * Try to give ourselves destroy rights.
	 */
	o->zo_access |= (NIS_DESTROY_ACC << 16);
	res = nis_modify(name, o);

	if (res == NULL)
		errx(1, "out of memory doing nis_modify");

	if (res->status != NIS_SUCCESS) {
		nis_freeresult(res);
		return;
	}

	nis_freeresult(res);
	
	res = nis_remove(name, NULL);

	if (res != NULL);
		nis_freeresult(res);

	return;
}

static int do_rm(name, i, f)
	nis_name		name;
	int			i, f;
{
	nis_result		*res;
	nis_object		*o;
	char			fullname[NIS_MAXNAMELEN];

	if (strchr(name, '[') != NULL)
		errx(1, "can't remove indexed names");

	res = nis_lookup(name, EXPAND_NAME);

	if (res == NULL)
		errx(1, "out of memory doing nis_lookup");

	if (res->status != NIS_SUCCESS) {
		if (res->status != NIS_PERMISSION || !f) {
			warnx("nis_lookup %s: %s", name,
					 nis_sperrno(res->status));
			nis_freeresult(res);
			return(1);
		}
	}

	o = NIS_RES_OBJECT(res);
#ifdef foo
	if (__type_of(o) == DIRECTORY_OBJ) {
		warnx("%s: is a directory!", name);
		nis_freeresult(res);
		return(1);
	}
#endif
	NIS_RES_OBJECT(res) = NULL;
	NIS_RES_NUMOBJ(res) = 0;
	nis_freeresult(res);
	
	snprintf(fullname, sizeof(fullname), "%s.%s", o->zo_name, o->zo_domain);

	if (i) {
		int			ch, first;
		fprintf(stderr, "remove %s? ", fullname);
		fflush(stderr);
		first = ch = getchar();
		while (ch != '\n' && ch != -1)
			ch = getchar();
		if (first != 'y') {
			nis_destroy_object(o);
			return(0);
		}
	}

	res = nis_remove(fullname, NULL);

	if (res == NULL || res->status != NIS_SUCCESS) {
		if (res == NULL) {
			nis_destroy_object(o);
			errx(1, "out of memory doing nis_remove");
		} else if (res->status == NIS_PERMISSION) {
			if (f)
				try_force(fullname, o);
		} else
			warnx("%s: %s", fullname, nis_sperrno(res->status));
	}

	nis_destroy_object(o);
	nis_freeresult(res);

	return(0);
}

int main(argc, argv)
	int			argc;
	char			*argv[];
{
	int			ch, i;
	int			interactive = 0, force = 0;
	char			*pname = argv[0];

	while((ch = getopt(argc, argv, "if")) != -1) {
		switch(ch) {
		case 'i':
			interactive++;
			break;
		case 'f':
			force++;
			break;
		default:
			usage(argv[0]);
			break;
		}
	}

	argc -= optind;
	argv += optind;

	if (!argc)
		usage(pname);

	for (i = 0; i < argc; i++) {
		if (argv[i][strlen(argv[i] - 1)] != '.')
			interactive++;
		do_rm(argv[i], interactive, force);
	}

	exit(0);
}
