/*
 * Copyright (c) 1996, 1997
 *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: nis_cache_main.c,v 1.8 1997/06/19 21:32:06 wpaul Exp wpaul $
 */

#include "nis_cache.h"
#include <stdio.h>
#include <stdlib.h> /* getenv, exit */
#include <unistd.h>
#include <rpc/pmap_clnt.h> /* for pmap_unset */
#include <string.h> /* strcmp */
#include <signal.h>
#include <sys/ttycom.h> /* TIOCNOTTY */
#ifdef __cplusplus
#include <sysent.h> /* getdtablesize, open */
#endif /* __cplusplus */
#include <memory.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <syslog.h>
#include <errno.h>
#include <err.h>
#include "nis_cache_extern.h"

#ifndef SIG_PF
#define	SIG_PF void(*)(int)
#endif

#ifdef DEBUG
#define	RPC_SVC_FG
#endif

#define	_RPCSVC_CLOSEDOWN 120

#ifndef lint
static const char rcsid[] = "@(#) $Header: /proj/mbone/nis/usr.sbin/nis_cachemgr/RCS/nis_cache_main.c,v 1.8 1997/06/19 21:32:06 wpaul Exp wpaul $ (Project Flatbush)";
static const char copyright[] = "@(#) Copyright (c) 1996, 1997\
	Bill Paul. All rights reserved.";
#endif /* not lint */

int _rpcpmstart;		/* Started by a port monitor ? */
static int _rpcfdtype;
		 /* Whether Stream or Datagram ? */
	/* States a server can be in wrt request */

#define	_IDLE 0
#define	_SERVED 1
#define	_SERVING 2

static int _rpcsvcstate = _IDLE;	 /* Set when a request is serviced */
char *progname = "nis_cachemgr";
int insecure = 0;
directory_obj *dir;
nis_bound_directory d;

#ifndef CACHEMGRSOCK
#define CACHEMGRSOCK "/var/run/cachemgrsock"
#endif

static
void _msgout(char* msg)
{
#ifdef RPC_SVC_FG
	if (_rpcpmstart)
		syslog(LOG_ERR, msg);
	else
		(void) fprintf(stderr, "%s\n", msg);
#else
	syslog(LOG_ERR, msg);
#endif
}

static void
closedown(int sig)
{
	if (_rpcsvcstate == _IDLE) {
		extern fd_set svc_fdset;
		static int size;
		int i, openfd;

		if (_rpcfdtype == SOCK_DGRAM)
			exit(0);
		if (size == 0) {
			size = getdtablesize();
		}
		for (i = 0, openfd = 0; i < size && openfd < 2; i++)
			if (FD_ISSET(i, &svc_fdset))
				openfd++;
		if (openfd <= 1)
			exit(0);
	}
	if (_rpcsvcstate == _SERVED)
		_rpcsvcstate = _IDLE;

	(void) signal(SIGALRM, (SIG_PF) closedown);
	(void) alarm(_RPCSVC_CLOSEDOWN/2);
}

static void usage()
{
	fprintf(stderr, "usage: %s\n", progname);
	fprintf(stderr, "\t-d [debug_level]\n");
	fprintf(stderr, "\t-i <initialize cache when starting up>\n");
	fprintf(stderr, "\t-m [max size of cache file (in pages) ]\n");
	fprintf(stderr, "\t-n <insecure mode - directory object signatures \
not checked>.\n");
	fprintf(stderr, "\t-s [initial size of cache file (in pages)]\n");
	fprintf(stderr, "\t-v <verbose - sends events to syslog>\n");

	exit(1);
}

main(argc, argv)
	int argc;
	char *argv[];
{
	register SVCXPRT *transp = NULL;
	int sock;
	int proto = 0;
	struct sockaddr_in saddr;
	int asize = sizeof (saddr);
	int ch;

	while ((ch = getopt(argc, argv, "uh")) != -1) {
		switch(ch) {
		case 'u':
			insecure++;
			break;
		case 'h':
		default:
			usage();
			break;
		}
	}

#ifdef DEBUG
	debug = 1;
#endif

	if (getsockname(0, (struct sockaddr *)&saddr, &asize) == 0) {
		int ssize = sizeof (int);

		if (saddr.sin_family != AF_INET)
			exit(1);
		if (getsockopt(0, SOL_SOCKET, SO_TYPE,
				(char *)&_rpcfdtype, &ssize) == -1)
			exit(1);
		sock = 0;
		_rpcpmstart = 1;
		proto = 0;
		openlog(progname, LOG_PID, LOG_DAEMON);
	} else {
		if (!debug) {
			if (daemon(0, 0)) {
				err(1, "cannot fork");
			}
			openlog(progname, LOG_PID, LOG_DAEMON);
		}

		sock = RPC_ANYSOCK;
		(void) pmap_unset(CACHEPROG, CACHE_VER_2);
	}

	if ((_rpcfdtype == 0) || (_rpcfdtype == SOCK_DGRAM)) {
		transp = svcudp_create(sock);
		if (transp == NULL) {
			_msgout("cannot create udp service.");
			exit(1);
		}
		if (!_rpcpmstart)
			proto = IPPROTO_UDP;
		if (!svc_register(transp, CACHEPROG, CACHE_VER_2, cacheprog_2, proto)) {
			_msgout("unable to register (CACHEPROG, CACHE_VER_2, udp).");
			exit(1);
		}
	}

	if ((_rpcfdtype == 0) || (_rpcfdtype == SOCK_STREAM)) {
		transp = svctcp_create(sock, 0, 0);
		if (transp == NULL) {
			_msgout("cannot create tcp service.");
			exit(1);
		}
		if (!_rpcpmstart)
			proto = IPPROTO_TCP;
		if (!svc_register(transp, CACHEPROG, CACHE_VER_2, cacheprog_2, proto)) {
			_msgout("unable to register (CACHEPROG, CACHE_VER_2, tcp).");
			exit(1);
		}
	}

#ifdef __FreeBSD__
	if ((_rpcfdtype == 0) || (_rpcfdtype == SOCK_STREAM)) {
		unlink(CACHEMGRSOCK);
		transp = svcunix_create(sock, 0, 0, CACHEMGRSOCK);
		if (transp == NULL) {
			_msgout("cannot create AF_UNIX service.");
			exit(1);
		}
		if (!_rpcpmstart)
			proto = 0;
		if (!svc_register(transp, CACHEPROG, CACHE_VER_2, cacheprog_2, proto)) {
			_msgout("unable to register (CACHEPROG, CACHE_VER_2, unix).");
			exit(1);
		}
		chmod(CACHEMGRSOCK, 0666);
	}
#endif

	if (transp == (SVCXPRT *)NULL) {
		_msgout("could not create a handle");
		exit(1);
	}
	if (_rpcpmstart) {
		(void) signal(SIGALRM, (SIG_PF) closedown);
		(void) alarm(_RPCSVC_CLOSEDOWN/2);
	}
/*
	nis_cache_init();
	nis_cache_open();
	nis_lock_cache();
	nis_cache_load();
	nis_get_cold_start(&dir);
	nis_cache_remove(dir->do_name);
	nis_cache_dump();
	nis_cache_add(dir);
	nis_cache_dump();
	nis_unlock_cache();
*/
	nis_cachebind_open("/tmp/NIS_SHARED_DIRCACHE");
	nis_get_cold_start(&dir);
	nis_cachebind_delete(dir->do_name, CB_REPLICA);
	bzero((char *)&d, sizeof(d));
	bcopy((char *)dir, (char *)&d.dobj, sizeof(directory_obj));
	nis_cachebind_insert(dir->do_name, CB_REPLICA, NULL, &d, NULL);

	svc_run();
	_msgout("svc_run returned");
	exit(1);
	/* NOTREACHED */
}
