/*
 * Copyright (c) 1996, 1997, 1998
 *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	$Id$
 */

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <rpcsvc/nis.h>
#include <rpcsvc/nislib.h>
#include <rpcsvc/nis_cache.h>
#include "nis_cache_extern.h"
#include "nis_extern.h"

#ifndef lint
static const char rcsid[] =
	"@(#) $Id: nis_cache_server.c,v 1.10 1997/06/19 21:32:06 wpaul Exp wpaul $";
#endif /* not lint */

void *
nis_cache_add_entry_2_svc(fd_result *argp, struct svc_req *rqstp)
{
	static char 		*result;
#ifdef notused
	directory_obj		*dir;
	XDR			xdrs;
#ifdef __FreeBSD__
	struct sockaddr_in	*rqhost;

	rqhost = svc_getcaller(rqstp->rq_xprt);
	if (rqhost->sin_family == AF_INET) {
		nis_err("request from non-local host rejected");
		return(NULL);
	}
#endif
	if (nis_verify_sig(argp)) {
		nis_err("directory signature verification failed!");
		return((void *) &result);
	}

	dir = (struct directory_obj *)calloc(1, sizeof(directory_obj));
	xdrmem_create(&xdrs, argp->dir_data.dir_data_val,
			argp->dir_data.dir_data_len, XDR_DECODE);
	xdr_directory_obj(&xdrs, dir);
	xdr_destroy(&xdrs);
	nis_lock_cache();
	nis_cache_add(dir);
	nis_cache_dump();
	nis_unlock_cache();
#endif
	return((void *) &result);
}

void *
nis_cache_remove_entry_2_svc(directory_obj *argp, struct svc_req *rqstp)
{
	static char * result;
#ifdef notused
#ifdef __FreeBSD__
	struct sockaddr_in	*rqhost;

	rqhost = svc_getcaller(rqstp->rq_xprt);
	if (rqhost->sin_family == AF_INET) {
		nis_err("request from non-local host rejected");
		return(NULL);
	}
#endif
	nis_lock_cache();
	nis_cache_remove(argp->do_name);
	nis_cache_dump();
	nis_unlock_cache();
#endif
	return((void *) &result);
}

void *
nis_cache_read_coldstart_2_svc(void *argp, struct svc_req *rqstp)
{
	static char * result;
#ifdef notused
#ifdef __FreeBSD__
	struct sockaddr_in	*rqhost;

	rqhost = svc_getcaller(rqstp->rq_xprt);
	if (rqhost->sin_family == AF_INET) {
		nis_err("request from non-local host rejected");
		return(NULL);
	}
#endif
	/*
	 * insert server code here
	 */
#endif
	return((void *) &result);
}

void *
nis_cache_refresh_entry_2_svc(char **argp, struct svc_req *rqstp)
{
	static char * result;
#ifdef notused
	nis_result		*res;
#ifdef __FreeBSD__
	struct sockaddr_in	*rqhost;
	
	rqhost = svc_getcaller(rqstp->rq_xprt);
	if (rqhost->sin_family == AF_INET) {
		nis_err("request from non-local host rejected");
		return(NULL);
	}
#endif

	nis_cache_remove(*argp);
	/*
	 * We must use NO_CACHE here since the cache manager
	 * isn't allowed to call itself. (We'd become deadlocked.)
	 */
	res = nis_lookup(*argp, MASTER_ONLY|NO_CACHE);
	if (res != NULL && res->status == NIS_SUCCESS) {
		if (__type_of(NIS_RES_OBJECT(res)) == DIRECTORY_OBJ) {
			nis_lock_cache();
			nis_cache_add(&NIS_RES_OBJECT(res)->DI_data);
			nis_cache_dump();
			nis_unlock_cache();
			nis_freeresult(res);
		}
	}
#endif
	return((void *) &result);
}

/*
 * The following procedures were added in Solaris 2.6. The idea, as I
 * (think) I understand it, is to have nis_cachemgr do the binding
 * gruntwork on behalf of the calling process rather than the process
 * doing the gruntwork and handing nis_cachemgr the result for safekeeping.
 * This allows us to do away with checking signatures in fd_result
 * structs without letting a local process poison the cache.
 */

/*
 * Establish a binding to a replica server of the specified domain.
 * It's unclear if we should force a binding to a replica or allow
 * bindings to both a master or a replica. What if no replicas are
 * available? We can do one of two things:
 *
 * o Bind to a master anyway
 * o Return a failure code and let the client call the
 *   'bind_master' procedure as a follow up.
 *
 * We do the latter, not knowing any better.
 */
nis_error *
nis_cache_bind_replica_2_svc(char **argp, struct svc_req *rqstp)
{
	static nis_error  result;
	struct _nis_binding	*bptr;
	nis_bound_directory	d;
	struct manual_binding	*m;

	result = __nis_bind((char *)*argp, NO_CACHE, &bptr);
	if (result != NIS_SUCCESS)
		return(&result);
	bzero((char *)&d, sizeof(d));
	m = bptr->nb_binding_manual;
	bcopy((char *)m->mb_dir, (char *)&d.dobj, sizeof(directory_obj));
	nis_cachebind_insert((char *)*argp, CB_REPLICA, NULL, &d, NULL);
	__nis_destroy_binding(bptr);

	return (&result);
}

/*
 * Return a binding to a master server for the specified domain.
 * Like the previous routine, if we fail, we return an error code
 * and let the caller roll over to the 'bind_replica' function.
 */
nis_error *
nis_cache_bind_master_2_svc(char **argp, struct svc_req *rqstp)
{
	static nis_error  result;
	struct _nis_binding	*bptr;
	nis_bound_directory	d;
	struct manual_binding	*m;

	result = __nis_bind((char *)*argp, MASTER_ONLY|NO_CACHE, &bptr);
	if (result != NIS_SUCCESS)
		return(&result);
	bzero((char *)&d, sizeof(d));
	m = bptr->nb_binding_manual;
	bcopy((char *)m->mb_dir, (char *)&d.dobj, sizeof(directory_obj));
	nis_cachebind_insert((char *)*argp, CB_MASTER, NULL, &d, NULL);
	__nis_destroy_binding(bptr);

	return (&result);
}

/*
 * Return a binding for the specified server structure. The trick
 * here is that while the client obviously knows what server it
 * wants to talk to, it may have multiple interfaces and hence
 * multiple endpoint structures. It's up to us to chose the
 * right one (for some value of 'right').
 */
nis_error *
nis_cache_bind_server_2_svc(bind_server_arg *argp, struct svc_req *rqstp)
{
	static nis_error  result;

	/*
	 * insert server code here
	 */

	return (&result);
}

/*
 * We have a binding for a directory which is stale or unsatisfactory
 * for some reason: update it.
 */
refresh_res *
nis_cache_refresh_binding_2_svc(nis_bound_directory *argp, struct svc_req *rqstp)
{
	static refresh_res  result;

	/*
	 * insert server code here
	 */

	return (&result);
}

/*
 * We have a binding for a server that's stale or unsatisfactory for
 * some reason: update that.
 */
refresh_res *
nis_cache_refresh_address_2_svc(nis_bound_endpoint *argp, struct svc_req *rqstp)
{
	static refresh_res  result;

	/*
	 * insert server code here
	 */

	return (&result);
}

/*
 * We have a callback endpoint that's stale or unsatisfactory for
 * some reason: update it.
 *
 * I'm uncertain as to why the cache manager would cache callback
 * endpoints. For one thing, each time a client generates a callback
 * endpoint, it does it using an ephemeral port number, which means
 * the uaddr is constantly changing.
 */
refresh_res *
nis_cache_refresh_callback_2_svc(nis_bound_endpoint *argp, struct svc_req *rqstp)
{
	static refresh_res  result;

	/*
	 * insert server code here
	 */

	return (&result);
}
