/*
 * Copyright (c) 1996, 1997, 1998
 *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	$Id$
 */

#include <sys/types.h>
#include <sys/cdefs.h>

#include <stdio.h>
#include <err.h>
#include <stdlib.h>
#include <unistd.h>

#include <rpcsvc/nis.h>
#include <rpcsvc/nislib.h>

#ifndef lint
static const char rcsid[] =
	"@(#) $Id$";
#endif

static void usage(pname)
	char			*pname;
{
	fprintf(stderr, "usage: %s [-M] [-s] [-v on|off] [-f dgot "
		"[-n obj_name]] [-H host] domain\n", pname);
	fprintf(stderr, "\t-M: heap memory\n");
	fprintf(stderr, "\t-s: statistics\n");
	fprintf(stderr, "\t-v: verbose mode\n");
	fprintf(stderr, "\t-f: flush cache (dir, group, object, table) "
			"[-n obj_name]\n");
	exit(1);
}

int main(argc, argv)
	int			argc;
	char			*argv[];
{
	nis_tag			tag = { 0, NULL };
	nis_tag			*tres;
	int			ch;
	nis_name		domain = nis_local_directory();
	nis_name		srvname = NULL;
	nis_result		*res;
	nis_error		r;
	nis_server		*s;
	directory_obj		*d;
	int			i;

	while((ch = getopt(argc, argv, "Msv:f:n:H:")) != -1) {
		switch(ch) {
		case 'M':
			tag.tag_type = TAG_HEAP;
			tag.tag_val = "";
			break;
		case 's':
			tag.tag_type = TAG_STATS;
			tag.tag_val = "";
			break;
		case 'v':
			tag.tag_type = TAG_DEBUG;
			tag.tag_val = optarg;
		case 'f':

		case 'n':
		case 'H':
			srvname = optarg;
			break;
		default:
			usage(argv[0]);
		}
	}
	argc -= optind;
	argv += optind;
	if (argv != NULL)
		domain = argv[0];

	res = nis_lookup(domain, 0);
	if (res == NULL)
		errx(1, "out of memory doing nis_lookup()");
	r = res->status;	
	if (r != NIS_SUCCESS) {
		nis_freeresult(res);
		errx(1, "%s: %s", domain, nis_sperrno(r));
	}
	if (__type_of(NIS_RES_OBJECT(res)) != DIRECTORY_OBJ) {
		nis_freeresult(res);
		errx(1, "%s is not a directory!");
	}
	d = &NIS_RES_OBJECT(res)->DI_data;
	for (i = 0; i < d->do_servers.do_servers_len; i++) {
		s = &d->do_servers.do_servers_val[i];
		if (srvname != NULL &&
			nis_dir_cmp(srvname, s->name) != SAME_NAME)
			continue;
		r = nis_servstate(s, &tag, 1, &tres);
		if (r != NIS_SUCCESS)
			warnx("Set server state failed : %s", nis_sperrno(r));
		else {
			printf("%s", tres->tag_val);
			nis_freetags(tres, 1);
		}
	}

	nis_freeresult(res);
	exit(0);
}
