/*
 * Copyright (c) 1996, 1997
 *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	$Id: nisstat.c,v 1.2 1998/07/18 20:41:35 wpaul Exp $
 */

#include <sys/types.h>
#include <sys/cdefs.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <err.h>
#include <errno.h>

#include <rpcsvc/nis.h>
#include <rpcsvc/nislib.h>
#include <rpcsvc/nis_tags.h>

#ifndef lint
static const char copyright[] = "@(#) Copyright (c) 1996, 1997\
	Bill Paul. All rights reserved.";
static const char rcsid[] =
	"@(#) $Id: nisstat.c,v 1.2 1998/07/18 20:41:35 wpaul Exp $ (Project Flatbush)";
#endif

void usage(pname)
	char			*pname;
{
	fprintf(stderr, "%s [-H host] [directory]\n", pname);
	exit(1);
}

#define NUMTAGS		11

int main(argc, argv)
	int			argc;
	char			*argv[];
{
	int			ch;
	nis_tag			t[NUMTAGS] = {
			{ TAG_ROOTSERVER, "" },
			{ TAG_NISCOMPAT, "" },
			{ TAG_DNSFORWARDING, "" },
			{ TAG_SECURITY_LEVEL, "" },
			{ TAG_DIRLIST, "" },
			{ TAG_OPSTATS, "" },
			{ TAG_S_DCACHE, "" },
			{ TAG_S_GCACHE, "" },
			{ TAG_S_STORAGE, "" },
			{ TAG_HEAP, "" },
			{ TAG_UPTIME, "" }
	};
	char			*descs[] = {
			"root server",
			"NIS compat mode",
			"DNS forwarding in NIS mode",
			"security level",
			"serves directories",
			"Operation Statistics",
			"directory cache",
			"group cache",
			"static storage",
			"dynamic storage",
			"up since"
	};
	nis_tag			*r;
	nis_result		*res;
	directory_obj		*d;
	nis_name		dir = nis_local_directory();
	int			i, j;
	nis_error		terr;
	nis_server		*srv;
	char			*srvname = NULL, *pname = argv[0];

	while((ch = getopt(argc, argv, "H:")) != -1) {
		switch(ch) {
		case 'H':
			srvname = optarg;
			break;
		default:
			usage(pname);
			break;
		}
	}

	argc -= optind;
	argv += optind;

	if (argc)
		dir = argv[0];

	res = nis_lookup(dir, EXPAND_NAME|FOLLOW_LINKS);

	if (res == NULL)
		errx(1, "out of memory doing nis_lookup");

	if (res->status != NIS_SUCCESS) {
		warnx("%s: %s", dir, nis_sperrno(res->status));
		nis_freeresult(res);
		exit(1);
	}

	if (__type_of(NIS_RES_OBJECT(res)) != DIRECTORY_OBJ) {
		warnx("%s is not a directory!", dir);
		nis_freeresult(res);
		exit(1);
	}

	printf("Statistics for domain %s :\n\n", dir);

	d = &NIS_RES_OBJECT(res)->DI_data;

	for (i = 0; i < d->do_servers.do_servers_len; i++) {
		srv = &d->do_servers.do_servers_val[i];
		if (srvname != NULL &&
			nis_dir_cmp(srv->name, srvname) != SAME_NAME)
			continue;
		terr = nis_stats(srv, t, NUMTAGS, &r);
		if (terr != NIS_SUCCESS) {
			warnx("%s: %s", srv->name, nis_sperrno(terr));
			continue;
		}
		printf("Statistics from server : %s\n", srv->name);
		for (j = 0; j < NUMTAGS; j++) {
			if (r[j].tag_type == TAG_DIRLIST ||
			    r[j].tag_type == TAG_OPSTATS) {
				char			*p, *q;
				printf("Stat: '%s' :\n", descs[j]);
				p = r[j].tag_val;
				while (*p != NUL) {
					q = strpbrk(p, "\n \t");
					if (q != NULL)
						*q = '\0';
					printf("\t%s\n", p);
					p = q;
					if (q == NULL)
						break;
					*q = '\n';
					p++;
				}
			} else
				printf("Stat: '%s' = '%s'\n", descs[j],
							      r[j].tag_val);
		}
		nis_freetags(r, NUMTAGS);
	}

	nis_freeresult(res);

	exit(0);
}
