/*
 * Copyright (c) 1996, 1997
 *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: nis_callback.c,v 1.1 1998/07/08 19:23:20 wpaul Exp wpaul $
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/queue.h>
#include <rpc/rpc.h>
#include <rpc/auth_unix.h>
#include <rpcsvc/nis.h>
#include <rpcsvc/nis_db.h>
#include <rpcsvc/nis_callback.h>
#include "nis_extern.h"

#ifndef lint
static const char rcsid[] =
	"@(#) $Id: nis_callback.c,v 1.1 1998/07/08 19:23:20 wpaul Exp wpaul $";
#endif

static nis_error nis_queue_callback __P((CLIENT *, int, nis_object *,
					uint32_t, nis_name, nis_name,
					uint32_t, netobj *,
				        SVCXPRT *, struct timeval * ));

extern AUTH *authdes_pk_seccreate __P(( char *, netobj *, unsigned int,
					char *, des_block *, nis_server * ));

/*
 * Do I really have to go through all this crap?
 */

/*
 * The first half of this file contains two callback initiator routines.
 * These are entry points to the callback system, which consists of a
 * queuing mechanism and a special non-blocking RPC tramsission mechanism.
 * The initiators open the connection to the client and transmit the first
 * entry directly. The callback instance is then added to a queue and
 * the initiator returns. When the client replies, the handler routine
 * handles the response and sends the next entry. This continues until
 * the client tells us to shut up or we run out of entries.
 *
 * The special RPC mess is used to allow splitting the normal client-side
 * RPC behavior into two parts: normally clnt_call() will send a request
 * and then block waiting for a reply. In our case, we want to send the
 * request and then return, letting select() notify us when the process
 * on the other end replies. This nonsense allows us to avoid deadlock
 * conditions in the callback system without using threads, which, as
 * we all know, are the work of satan.
 */

static LIST_HEAD(head, callentry) callhead;

#define CBTYPE_BATCH	1		/* send pre-loaded objects */
#define CBTYPE_ALL	2		/* dump whole table */
#define CBTYPE_DUMP	3		/* dump whole directory */
#define CB_CLUSTERSZ	50		/* # of obj to xmit at a time */
#define CB_TTL		10		/* time to live */

struct callentry {
	CLIENT			*clnt;		/* client handle */
	SVCXPRT			*xprt;		/* server transport */
	struct timeval		ticks;		/* ticks from upper level */
	nis_name		name;		/* table name */
	nis_name		tname;		/* altername name */
	nis_name		principal;	/* client principal name */
	nis_object		*objs;		/* pre-read objects */
	netobj			cookie;		/* callback ID cookie */
	netobj			next;		/* all/dump next entry ptr */
	netobj			save;		/* saved dump table ptr */
	unsigned int		numobjs;	/* counter */
	unsigned int		cnt;		/* other counter */
	uint32_t		flags;		/* i-base semantics flags */
	nis_error		stat;		/* callback status */
	int			cbtype;		/* callback type */
	int			ttl;		/* time to live */
	int			exiting;	/* cb is being shut down */
	LIST_ENTRY(callentry)	links;
};

static bool_t nis_xmit_cb_all(q)
	register struct callentry *q;
{
	int			i = 0;	/* must not be unsigned! */
	cback_data		cb_arg;
	nis_result		res;
	nis_error		cerr;
	nis_object		*objarray[CB_CLUSTERSZ], *o = NULL;

	bzero((char *)&res, sizeof(res));

	/*
	 * Sanity check: can only dump tables.
	 */
	if (q->xprt != NULL) {
		cerr = __nis_ocache_lookup(q->name, 0, 0, &o);
		if (cerr != NIS_SUCCESS) {
			res.status = NIS_CBRESULTS;
			res.zticks = __stop_clock(&q->ticks);
			svc_sendreply(q->xprt, xdr_nis_result, (char *)&res);
			cbproc_error_1(&cerr, q->clnt);
			q->stat = cerr;
			q->xprt = NULL;
			return(FALSE);
		}

		if (__type_of(o) != TABLE_OBJ &&
			__type_of(o) != LINK_OBJ) {
			res.status = NIS_NOCALLBACK;
			res.zticks = __stop_clock(&q->ticks);
			NIS_RES_OBJECT(&res) = o;
			NIS_RES_NUMOBJ(&res) = 1;
			svc_sendreply(q->xprt, xdr_nis_result, (char *)&res);
			NIS_RES_OBJECT(&res) = NULL;
			NIS_RES_NUMOBJ(&res) = 0;
			q->xprt = NULL;
			q->stat = cerr;
			return(FALSE);
		}
	}

	for (i = 0; i < q->cnt; i++) {
		bzero((char *)&res, sizeof(res));
		if (q->next.n_len) {
			res.status = nis_db_ibnext(q->name, q->flags,
						   &q->next, &res,
						   q->principal);
		} else {
			res.status  = nis_db_ibfirst(q->name, q->flags,
						     0, NULL, &res,
						     q->principal);
			if (q->xprt != NULL) {
				cerr = res.status;
				res.status = NIS_CBRESULTS;
				res.zticks = __stop_clock(&q->ticks);
				svc_sendreply(q->xprt, xdr_nis_result,
						 (char *)&res);
				q->xprt = NULL;
				res.status = cerr;
			}
		}
		q->next.n_len = 0;
		q->next.n_bytes = NULL;
		if (res.status != NIS_SUCCESS)
			break;
		objarray[i] = NIS_RES_OBJECT(&res);
		q->next.n_len = res.cookie.n_len;
		q->next.n_bytes = res.cookie.n_bytes;
		NIS_RES_OBJECT(&res) = NULL;
		NIS_RES_NUMOBJ(&res) = 0;
		res.cookie.n_len = 0;
		res.cookie.n_bytes = NULL;
		xdr_free(xdr_nis_result, (char *)&res);
	}

	/*
	 * How many objects were read
	 */
	q->numobjs = i;
	if (i) {
		cb_arg.entries.entries_len = i;
		cb_arg.entries.entries_val = objarray;
		cbproc_receive_1_send(&cb_arg, q->clnt);
	} else {
		/*
		 * XXX If we were called bu the dump transmitter,
		 * don't return an error here. Let the dump transmitter
		 * do it.
	 	 */
		if (!q->save.n_len) {
			if (res.status != NIS_SUCCESS &&
				res.status != NIS_NOTFOUND) {
				q->stat = res.status;
				cbproc_error_1(&res.status, q->clnt);
			}
		}
	}

	cerr = res.status;
	for (i = 0; i < q->numobjs; i++) {
		xdr_free(xdr_nis_object, (char *)objarray[i]);
		free(objarray[i]);
		objarray[i] = NULL;
	}

	if (cerr != NIS_SUCCESS)
		return(FALSE);

	return(TRUE);
}

/*
 * This callback transmitter is designed for use by the nis_dump
 * procedure. It has to not only dump the contents of a directory, it
 * also has to dump the contents of the tables the reside in the
 * directory. You might think this is the sort of thing best done
 * with recursion, but this isn't really true since we're only going
 * one level deep into the directory. We start off as though we are
 * merely dumping the directory alone, but if we encounter a TABLE_OBJ,
 * we switch gears and start dumping that instead. Once we hit the end
 * of the table, we revert back to dumping the directory (until we
 * encounter another table). The trick is to break this up so that we
 * maintain our train of thought while transmitting only CB_CLUSTERSZ
 * entries at a time.
 */
static bool_t nis_xmit_cb_dump(q)
	register struct callentry *q;
{
	int			i = 0;	/* must not be unsigned! */
	cback_data		cb_arg;
	nis_error		err;
	nis_object		*objarray[CB_CLUSTERSZ];
	bool_t			r;
	char			tbuf[NIS_MAXNAMELEN];
	int				stop = 0;

	if (q->xprt != NULL) {
		q->xprt = NULL;
	}

	/*
	 * The existence of a 'save' cookie indicates that we
	 * previously encountered a table within the directory and
	 * are dumping that now. The 'next' cookie is the next entry
	 * in the table, while the 'save' cookie is the next entry
	 * in the directory. We will return to reading the directory
	 * once we run out of entries in the table.
	 */
	if (q->save.n_len) {
		r = nis_xmit_cb_all(q);
		if (r == TRUE && q->numobjs)
			return(TRUE);
		else {
			q->next.n_len = q->save.n_len;
			q->next.n_bytes = q->save.n_bytes;
			q->save.n_len = 0;
			q->save.n_bytes = NULL;
			free(q->name);
			q->name = q->tname;
			q->tname = NULL;
			if (r == FALSE && q->numobjs)
				return(TRUE);
		}
	}

	for (i = 0; i < q->cnt; i++) {
		db_status			db_stat;
		struct db_objs			dres;

		if (stop)
			break;

		if (q->next.n_len)
			db_stat = db_next(q->name, &q->next, &dres);
		else
			db_stat = db_first(q->name, NULL, 0, &dres);

		err = db_to_nis_err(db_stat);
		if (err != NIS_SUCCESS)
			break;

		objarray[i] = NIS_RES_OBJECT(&dres);
		q->next.n_len = dres.cookie.n_len;
		q->next.n_bytes = dres.cookie.n_bytes;

		snprintf(tbuf, sizeof(tbuf), "%s.%s", objarray[i]->zo_name,
						objarray[i]->zo_domain);
		if (__type_of(objarray[i]) == TABLE_OBJ) {
			q->save.n_len = q->next.n_len;
			q->save.n_bytes = q->next.n_bytes;
			q->next.n_len = 0;
			q->next.n_bytes = NULL;
			q->cnt = CB_CLUSTERSZ - i;
			q->tname = q->name;
			q->name = strdup(tbuf);
			stop++;
		}
	}

	q->numobjs = i;

	if (i) {
		cb_arg.entries.entries_len = i;
		cb_arg.entries.entries_val = objarray;
		cbproc_receive_1_send(&cb_arg, q->clnt);
	} else {
		if (err != NIS_NOTFOUND) {
			q->stat = err;
			cbproc_error_1(&err, q->clnt);
		}
	}

	for (i = 0; i < q->numobjs; i++) {
		xdr_free(xdr_nis_object, (char *)objarray[i]);
		free(objarray[i]);
	}

	if (err != NIS_SUCCESS)
		return(FALSE);

	return(TRUE);
}

static bool_t nis_xmit_cb_batch(q)
	register struct callentry *q;
{
	nis_object		*objarray[CB_CLUSTERSZ];
	uint32_t		i;
	cback_data		cb_arg;

	if (q->xprt != NULL) {
		q->xprt = NULL;
	}

	for (i = 0; i < CB_CLUSTERSZ; i++) {
		if (q->cnt == q->numobjs)
			break;
		objarray[i] = &q->objs[i];
		q->cnt++;
	}

	if (i) {
		cb_arg.entries.entries_len = i;
		cb_arg.entries.entries_val = objarray;
		cbproc_receive_1_send(&cb_arg, q->clnt);
	}

	return(i ? TRUE : FALSE);
}

static nis_error __nis_make_cb_handle(client, rqstp, handle)
	nis_server		*client;
	struct svc_req		*rqstp;
	CLIENT			**handle;
{
	struct sockaddr_in	sin;
	CLIENT			*clnt;
	int 			sock = RPC_ANYSOCK;
	struct timeval		timeout = { 0 , 0 };
	int			s, nodelay = 1;
#ifdef USE_SECURE_RPC
	struct authdes_cred	*cred;
#endif

	bzero((char *)&sin, sizeof(struct sockaddr_in));

	if (__nis_uaddr_to_sockaddr(client->ep.ep_val, &sin) != NIS_SUCCESS) {
		nis_err("address conversion failed during callback");
		return(NIS_CBERROR);
	}

	if (strcmp(client->ep.ep_val->proto, "udp")) {
		clnt = clnttcp_create(&sin, CB_PROG, CB_VERS, &sock, 0, 0);
		if (clnt == NULL) {
			nis_err("%s", clnt_spcreateerror("tcp cb handle"));
			return(NIS_CBERROR);
		}
		clnt->cl_ops->cl_call = clnttcp_a_call;
	} else {
		clnt = clntudp_create(&sin, CB_PROG, CB_VERS, timeout, &sock);
		if (clnt == NULL) {
			nis_err("%s", clnt_spcreateerror("udp cb handle"));
			return(NIS_CBERROR);
		}
		clnt->cl_ops->cl_call = clntudp_a_call;
	}


	switch(rqstp->rq_cred.oa_flavor) {
#ifdef USE_SECURE_RPC
	case AUTH_DES:
		if (!client->pkey.n_len || client->pkey.n_bytes == NULL) {
			cred = (struct authdes_cred *)rqstp->rq_clntcred;
			clnt->cl_auth =
				authdes_pk_seccreate(cred->adc_fullname.name,
					  &client->pkey, NIS_WINDOW, NULL,
					  NULL, client);
			break;
		}
		/* else fallthrough */
#endif
	case AUTH_UNIX:
		clnt->cl_auth = authunix_create_default();
		break;
	default:
		clnt->cl_auth = authnone_create();
		break;
	}

	if (clnt->cl_auth == NULL) {
		nis_err("failed to create RPC creds in callback!");
		clnt_destroy(clnt);
		return(NIS_CLNTAUTH);
	}

	clnt_control(clnt, CLSET_TIMEOUT, &timeout);


	/*
	 * If this is a TCP connection, de-Nagle it. This improves
	 * interactive response somewhat.
	 */
	if (strcmp(client->ep.ep_val->proto, "udp")) {
		clnt_control(clnt, CLGET_FD, &s);
		if (setsockopt(s, IPPROTO_TCP, TCP_NODELAY,
			(void *)&nodelay, sizeof(nodelay)) == -1)
			nis_err("failed to turn off delay on socket: %s",
							strerror(errno));
	}

	*(caddr_t *)handle = (caddr_t)clnt;

	return(NIS_SUCCESS);
}

/*
 * The 'batch' callback function sends a single nis_result with zero
 * or more nis_objects hung off it. In other words, all data is sent
 * in a single RPC rather than sending one nis_object at a time.
 */
nis_error nis_callback_batch(objs, numobjs, client, error, rqstp, cookie)
	nis_object		*objs;
	uint32_t		numobjs;
	nis_server		*client;
	nis_error		error;
	struct svc_req		*rqstp;
	netobj			*cookie;
{
	CLIENT			*clnt;
	nis_error		stat;

	stat = __nis_make_cb_handle(client, rqstp, &clnt);

	if (stat != NIS_SUCCESS)
		return(stat);

	if (error) {
		cbproc_error_1(&error, clnt);
		auth_destroy(clnt->cl_auth);
		clnt_destroy(clnt);
		return(NIS_SUCCESS);
	}

	return(nis_queue_callback(clnt, CBTYPE_BATCH, objs, numobjs,
				NULL, NULL, 0, cookie, rqstp->rq_xprt, NULL));
}

/*
 * This is similar to what ypserv would do for a yp_all(): we keep
 * transmitting entries to the caller until we run out or the caller
 * tells us to shut up.
 */
nis_error nis_callback_all(client, name, principal, flags, rqstp, cookie, ticks)
	nis_server		*client;
	nis_name		name;
	nis_name		principal;
	uint32_t		flags;
	struct svc_req		*rqstp;
	netobj			*cookie;
	struct timeval		*ticks;
{
	CLIENT			*clnt;
	nis_error		stat;

	stat = __nis_make_cb_handle(client, rqstp, &clnt);

	if (stat != NIS_SUCCESS)
		return(stat);

	return(nis_queue_callback(clnt, CBTYPE_ALL, NULL, 0, name,
				  principal, flags, cookie,
				  rqstp->rq_xprt, ticks));

}

/*
 * This routine handles callbacks for the nis_dump() routine,
 * which sends both the contents of a directory and the contents
 * of all tables in the directory.
 */
nis_error nis_callback_dump(client, name, principal, rqstp, cookie)
	nis_server		*client;
	nis_name		name;
	nis_name		principal;
	struct svc_req		*rqstp;
	netobj			*cookie;
{
	CLIENT			*clnt;
	nis_error		stat;

	stat = __nis_make_cb_handle(client, rqstp, &clnt);

	if (stat != NIS_SUCCESS)
		return(stat);

	return(nis_queue_callback(clnt, CBTYPE_DUMP, NULL, 0,
				  name, principal, 0, cookie,
				  rqstp->rq_xprt, NULL));
}

/*
 * The second half of this file deals with the scheduling of callbacks.
 * Here's a naughty little secret for you: NIS+ was designed with threads
 * in mind. For example, a client may make a call to us that results in
 * a callback, and may have specified a callback function to nis_list().
 * The nis_list() routine will invoke the client's callback routine each
 * time it gets a result from the server, and then the client might turn
 * around and make _another_ NIS+ call inside its callback routine. If you
 * don't deal with this carefully, you could deadlock: the client will be
 * trying to talk to us while we're blocked inside clnt_call() in the
 * server's callback function.
 * 
 * To deal with this, we have a magic version of clnt_call() that lets us
 * transmit an RPC and return immediately without getting blocked waiting
 * for an answer. We can then send the socket into our select() loop and
 * read back the response when it's ready.
 */

nis_error nis_init_callbacks()
{
	LIST_INIT(&callhead);
	return(NIS_SUCCESS);
}

static void nis_callback_destroy(q)
	struct callentry	*q;
{
	if (q == NULL)
		return;

	LIST_REMOVE(q, links);

	if (q->clnt != NULL) {
		auth_destroy(q->clnt->cl_auth);
		clnt_destroy(q->clnt);
	}
	if (q->name != NULL)
		free(q->name);
	if (q->principal != NULL)
		free(q->principal);
	if (q->cookie.n_len)
		free(q->cookie.n_bytes);
	free(q);

	return;
}

static nis_error nis_handle_cb_dump(q)
	struct callentry	*q;
{
	bool_t			*cb_resp = NULL;
	cback_data		cb_arg;
	bool_t			r = FALSE;

	if (q->exiting)
		return(NIS_SUCCESS);

	if (q->stat != NIS_SUCCESS) {
		bzero((char *)&cb_arg, sizeof(cb_arg));
		cb_resp = cbproc_receive_1_recv(&cb_arg, q->clnt);
		if (cb_resp == NULL || *cb_resp == TRUE) {
		nis_db_ibendnext(q->name, q->flags,
					 &q->next, q->principal);
			return(NIS_SUCCESS);

		}
	}

	q->stat = NIS_CBRESULTS;
	if (q->cbtype == CBTYPE_DUMP)
		r = nis_xmit_cb_dump(q);
	if (q->cbtype == CBTYPE_ALL)
		r = nis_xmit_cb_all(q);

	if (r == FALSE) {
		if (q->stat == NIS_CBRESULTS)
			cbproc_finish_1(NULL, q->clnt);
		q->exiting++;
	}

	return(NIS_CBRESULTS);
}

static nis_error nis_handle_cb_batch(q)
	struct callentry	*q;
{
	cback_data		cb_arg;
	bool_t			*cb_resp = NULL;

	if (q->exiting)
		return(NIS_SUCCESS);

	if (q->stat != NIS_SUCCESS) {
		cb_resp = cbproc_receive_1_recv(&cb_arg, q->clnt);
		if (cb_resp == NULL) {
			return(NIS_SUCCESS);
		}
	}
	q->stat = NIS_CBRESULTS;

	if (nis_xmit_cb_batch(q) == FALSE) {
		cbproc_finish_1(NULL, q->clnt);
		q->exiting = 1;
	}

	return(NIS_CBRESULTS);
}

static bool_t nis_callback_cb(args, id, reason)
	void			*args;
	uint32_t		id;
	int			reason;
{
	register struct callentry *q;
	bool_t			r = FALSE;

	q = (struct callentry *)args;

	/* Timeout expired */
	if (!reason) {
		q->ttl--;
		if (q->ttl)
			return(TRUE);
		else
			return(FALSE);
	}

	switch(q->cbtype) {
	case CBTYPE_ALL:
	case CBTYPE_DUMP:
		q->ttl = CB_TTL;
		if (nis_handle_cb_dump(q) == NIS_SUCCESS)
			r = FALSE;
		else
			r = TRUE;
		break;
	case CBTYPE_BATCH:
		q->ttl = CB_TTL;
		if (nis_handle_cb_batch(q) == NIS_SUCCESS)
			r = FALSE;
		else
			r = TRUE;
		break;
	default:
		r = FALSE;
		break;
	}

	return(r);
}

static nis_error
nis_queue_callback(clnt, type, obj, cnt, name,
		   principal, flags, cookie, xprt, ticks)
	CLIENT			*clnt;
	int			type;
	nis_object		*obj;		/* Pointer to objects */
	uint32_t		cnt;		/* Number of objects */
	nis_name		name;		/* Table name */
	nis_name		principal;	/* Principal */
	uint32_t		flags;
	netobj			*cookie;
	SVCXPRT			*xprt;
	struct timeval		*ticks;
{
	register struct callentry *q = NULL;
	int			sock;

	if ((q = calloc(1,sizeof(struct callentry))) == NULL) {
		nis_err("out of memory allocating callback queue entry");
		return(NIS_NOMEMORY);
	}

	if (cookie != NULL) {
		q->cookie.n_bytes = cookie->n_bytes;
		q->cookie.n_len = cookie->n_len;
	}

	q->stat = NIS_SUCCESS;
	q->clnt = clnt;
	q->cbtype = type;
	q->ttl = CB_TTL;
	q->xprt = xprt;
	if (ticks != NULL)
		bcopy((char *)ticks, (char *)&q->ticks, sizeof(struct timeval));

	if (name == NULL) {
		/* This is a batch request */
		q->numobjs = cnt;
		q->objs = obj;
	} else {
		/* This is a dump request */
		q->name = strdup(name);
		q->principal = strdup(principal);
		q->cnt = CB_CLUSTERSZ;
		q->numobjs = 0;
		q->flags = flags;
	}

	clnt_control(q->clnt, CLGET_FD, &sock);
	nis_event_add_sock(nis_callback_cb, q, nis_callback_destroy,
				20, sock, "callback handler");
	LIST_INSERT_HEAD(&callhead, q, links);
	nis_callback_cb((void *)q, 0, 1);

	return(NIS_SUCCESS);
}

nis_error nis_callback_status(cookie)
	netobj *cookie;
{
	register struct callentry *q = NULL;

	for (q = callhead.lh_first; q != NULL; q = q->links.le_next) {
		if (q->cookie.n_len != cookie->n_len)
			continue;
		if (!bcmp(q->cookie.n_bytes, cookie->n_bytes, cookie->n_len))
			break;
	}

	if (q == NULL)
		return(NIS_NOTFOUND);

	return(q->stat);
}
