/*
 * Copyright (c) 1996, 1997
 *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: nis_cksum.c,v 1.1 1998/07/08 19:23:20 wpaul Exp wpaul $
 */

/*
 * The NIS+ server may sign a directory object by computing its
 * MD5 checksum and then encrypting it with ecb_crypt() using its
 * RPC private key. The checksum can then be decoded by nis_cachemgr
 * using the server's public key.
 *
 * It so happens that the MD5 library is included with the base
 * FreeBSD OS distribution, so we don't have to go looking for it.
 */

#include <sys/types.h>
#include <stdio.h>

#if defined(__sgi) || !defined(USE_SECURE_RPC)
struct signature *__nis_calculate_encrypted_cksum(data, len, pkey, mode)
	unsigned char		*data;
	unsigned int		len;
	char			*pkey;
	int			mode;
{
	return(NULL);
}
#else
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <md5.h>
#include <rpc/rpc.h>
#include <rpc/des_crypt.h>
#include <rpc/xdr.h>
#include <rpcsvc/nis.h>
#include "nis_extern.h"

#ifndef lint
static const char rcsid[] = "@(#) $Id: nis_cksum.c,v 1.1 1998/07/08 19:23:20 wpaul Exp wpaul $";
#endif

#define DIGEST_LEN 16

struct signature *__nis_calculate_encrypted_cksum(data, len, pkey, mode)
	unsigned char		*data;
	unsigned int		len;
	char			*pkey;
	int			mode;
{
	MD5_CTX			ctx;
	static unsigned char	digest[DIGEST_LEN];
	static struct signature	sig;
	des_block		deskey;
	int			cryptstatus;

	MD5Init(&ctx);
	MD5Update(&ctx,data,len);
	MD5Final(digest, &ctx);

	if (key_get_conv(pkey, &deskey) != 0)
		return(NULL);

	des_setparity((char *)&deskey);

	cryptstatus = ecb_crypt((char *)&deskey, (char *)&digest,
					 DIGEST_LEN, mode);

	if (DES_FAILED(cryptstatus))
		return(NULL);

	sig.signature_len = DIGEST_LEN;
	sig.signature_val = (char *)&digest;

	return(&sig);
}
#endif
