/*
 * Copyright (c) 1996, 1997, 1998
 *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: nis_extern.h,v 1.1 1998/07/08 19:23:20 wpaul Exp wpaul $
 */

#include <sys/types.h>
#include <sys/cdefs.h>
#include <sys/socket.h>

#include <netinet/in.h>
#include <arpa/inet.h>
#ifdef USE_SECURE_RPC
#include <rpc/des_crypt.h>
#include <rpc/key_prot.h>
#endif
#include <rpcsvc/nis.h>
#include <rpcsvc/nis_db.h>
#include <rpcsvc/nis_callback.h>
/*
#include "/proj/mbone/nis/lib/dbmalloc/malloc.h"
*/

#ifndef lint
static const char rcsid_nis_extern_h[] =
	"@(#) $Id: nis_extern.h,v 1.1 1998/07/08 19:23:20 wpaul Exp wpaul $";
#endif

#ifndef _PATH_NIS
#define _PATH_NIS "/var/nis"
#endif

#ifndef _PATH_NIS_DATA
#define _PATH_NIS_DATA "/var/nis/data"
#endif

#ifndef _NIS_CATALOG
#define _NIS_CATALOG "../data.dict"
#endif

#ifndef _PATH_NIS_TRANS_LOG
#define _PATH_NIS_TRANS_LOG "/var/nis/trans.log"
#endif

#ifndef _PATH_NIS_ROOT_OBJ
#define _PATH_NIS_ROOT_OBJ "/var/nis/data/root.object"
#endif

#ifndef _PATH_NIS_SRVLIST
#define _PATH_NIS_SRVLIST "/var/nis/data/serving_list"
#endif

#ifndef ROOT_OBJ
#define ROOT_OBJ "root.object"
#endif

#define sameobj(a, b)						\
	(a->zo_oid.ctime == b->zo_oid.ctime &&			\
	 a->zo_oid.mtime == b->zo_oid.mtime) ? 1 : 0

#define OBJ_VAL(PTR,O,C) \
 PTR->objects.objects_val[O]->en_cols.en_cols_val[C].ec_value.ec_value_val
#define OBJ_LEN(PTR,O,C) \
 PTR->objects.objects_val[O]->en_cols.en_cols_val[C].ec_value.ec_value_len
#define OBJ_FLAGS(PTR,O,C) \
 PTR->objects.objects_val[O]->en_cols.en_cols_val[C].ec_flags

#define ENTRY_TYPE(PTR) \
	PTR->objects.objects_val[0]->en_type

#define ENTRY_FLAGS(obj, col) \
	(obj)->EN_data.en_cols.en_cols_val[col].ec_flags

#ifndef NIS_WINDOW
#define NIS_WINDOW 300
#endif

struct entryattrs {
	unsigned int cnt;
	nis_attr *attrs;
};

struct db_objs {
	struct {
		unsigned int		objects_len;
		nis_object		*objects_val;
	} objects;
	netobj			cookie;
	uint32_t		dticks;
};

extern int		debug;
extern char		*progname;
extern int		securelevel;
extern time_t		boottime;
extern int		verbose;
extern int		readonly;
extern int		__nis_child_must_die;
extern char		**srvlist;
extern int		srvcnt;
extern nis_object	*root_obj;
extern nis_name		rootname;
extern time_t		nis_master_utime;
extern time_t		nis_replica_utime;

extern nis_object *nis_decode_nis_obj __P(( char * , uint32_t ));
extern char *nis_encode_nis_obj __P(( nis_object *, uint32_t * ));
extern char *nis_encode_directory_obj __P(( directory_obj *, uint32_t * ));
extern void nis_prog_3 __P((struct svc_req * , register SVCXPRT * ));
extern void nis_err  __P((const char * , ... ));

extern bool_t __is_local_dirname __P(( nis_name ));
extern bool_t __i_am_master __P(( directory_obj * ));
extern bool_t __i_am_replica __P(( directory_obj * ));
extern bool_t __is_local_dir __P(( directory_obj * ));
extern nis_error __nis_check_master __P(( nis_name ));

/*
 * Namespace routines.
 */

extern nis_error nis_db_nslookup __P(( nis_name, nis_result *, nis_name ));

extern nis_error nis_db_nsop __P(( nis_name, nis_object *, nis_name,
				   uint32_t, nis_result * ));

/*
 * Information base routines.
 */
extern nis_error nis_db_iblist __P(( nis_name, uint32_t, uint32_t,
					nis_attr *, nis_result *, nis_name ));
extern nis_error nis_db_ibfirst __P(( nis_name, uint32_t, uint32_t,
					nis_attr *, nis_result *, nis_name ));
extern nis_error nis_db_ibnext __P(( nis_name, uint32_t, netobj *,
						nis_result *, nis_name ));
extern nis_error nis_db_ibendnext __P(( nis_name, uint32_t, netobj *,
						nis_name ));

extern nis_error nis_db_ibop __P(( nis_name, uint32_t, uint32_t,
					nis_attr *, nis_object *, nis_name ,
					uint32_t, nis_result * ));
/*
 * Authentication routines.
 */
extern nis_name nis_principal __P(( struct svc_req * , char * ));
extern nis_error nis_can_read __P(( nis_name, nis_object * ));
extern nis_error nis_can_create __P(( nis_name, nis_object * ));
extern nis_error nis_can_modify __P(( nis_name, nis_object * ));
extern nis_error nis_can_delete __P(( nis_name, nis_object * ));
extern bool_t nis_check_access __P(( uint32_t, uint32_t,
					nis_name, nis_name, nis_name ));
extern nis_error getkeys_nisplus __P(( char *, char *, char *, char * ));

/*
 * Server state routines.
 */

extern nis_error nis_load_srvlist __P(( void ));
extern nis_error nis_dump_srvlist __P(( void ));
extern nis_error nis_add_to_srvlist __P(( nis_name ));
extern nis_error nis_remove_from_srvlist __P(( nis_name ));
extern bool_t nis_is_in_srvlist __P(( nis_name ));
extern nis_name nis_enum_srvlist __P(( nis_name ));

/*
 * Directory cache functions.
 */

#define	CHECK_MASTER	1	/* return NIS_NOTMASTER if not master */
#define CHECK_SERVER	2	/* return NIS_NOT_ME if not server */
#define CACHE_ONLY	4	/* only check cache */
#define LOOKUP_ONLY	8	/* only call nis_lookup(3) */

extern void __nis_ocache_insert __P(( nis_object * ));
extern nis_error __nis_dcache_lookup __P(( nis_name, uint32_t,
					 uint32_t, directory_obj ** ));
extern nis_error __nis_ocache_lookup __P(( nis_name, uint32_t,
					 uint32_t, nis_object ** ));
extern void __nis_init_ocache __P(( void ));
extern void __nis_get_dcache_stats __P(( uint32_t * ,
					uint32_t * ,
					uint32_t * ));
extern nis_error __nis_flush_dcache __P(( void ));
extern void __nis_flush_one_dir __P(( nis_name ));
extern nis_error __nis_flush_ocache __P(( void ));

/*
 * Misc
 */

extern directory_obj *__copy_dir __P(( directory_obj * ));
extern nis_name __nis_table __P(( nis_name, char * ));
extern void nis_init_cred_cache __P(( void ));
extern nis_error db_to_nis_err __P(( db_status ));
extern nis_error nis_get_root_obj __P(( nis_object ** ));
extern nis_error nis_put_root_obj __P(( nis_object * ));
extern nis_name __nis_find_root __P(( void ));
extern nis_error nis_empty_directory __P(( nis_name ));
extern void nis_free_dobjs __P(( struct db_objs * ));

extern nis_error nis_validname __P(( char * ));
extern nis_error nis_make_cookie __P(( struct svc_req *, netobj * ));
extern void __nis_unregister_xprt __P(( void ));
extern void __nis_register_xprt __P(( void ));
extern nis_error nis_create_readonly_child __P(( int * ));
extern nis_error nis_destroy_readonly_child __P(( int ));
extern char *nis_ctime __P(( time_t ));
extern enum auth_stat _svcauth_des_nisplus __P(( register struct svc_req * ,
						 register struct rpc_msg * ));
/*
 * Callback routines.
 */
extern nis_error nis_callback_batch __P(( nis_object * , uint32_t,
					  nis_server *, nis_error,
					  struct svc_req *, netobj * ));
extern nis_error nis_callback_dump __P(( nis_server *, nis_name, nis_name,
					struct svc_req *, netobj * ));
extern nis_error nis_callback_all __P(( nis_server *, nis_name, nis_name,
					uint32_t, struct svc_req *,
					netobj *, struct timeval * ));
extern nis_error nis_callback_status __P(( netobj * ));
extern nis_error nis_init_callbacks __P(( void ));
extern int nis_set_callback_fds __P(( fd_set * ));
extern void nis_run_callbackq __P(( fd_set * ));
extern void *cbproc_finish_1_send __P(( void *, CLIENT * ));
extern bool_t *cbproc_receive_1_send __P(( cback_data *, CLIENT * ));
extern bool_t *cbproc_receive_1_recv __P(( cback_data *, CLIENT * ));
extern bool_t nis_cbproc_receive_send __P(( cback_data *, CLIENT * ));
extern bool_t nis_cbproc_receive_recv __P(( cback_data *, CLIENT * ));
extern enum clnt_stat clntudp_a_call __P(( CLIENT *, uint32_t, xdrproc_t,
					   caddr_t, xdrproc_t, caddr_t,
						struct timeval ));
extern enum clnt_stat clnttcp_a_call __P(( CLIENT *, uint32_t, xdrproc_t,
					   caddr_t, xdrproc_t, caddr_t,
						struct timeval ));


/*
 * Database wrapper routines.
 */

extern db_status db_add __P(( nis_name, nis_object *, struct db_objs * ));
extern db_status db_remove __P(( nis_name , struct db_objs * ));
extern db_status db_newdir __P(( nis_name ));
extern db_status db_newtab __P(( nis_name , nis_object * ));
extern db_status db_rm __P(( nis_name ));
extern db_status db_exists __P(( nis_name ));
extern db_status db_lookup __P(( nis_name, struct db_objs * ));

extern db_status db_iblookup __P(( nis_name, nis_attr *, uint32_t ,
					struct db_objs * ));
extern db_status db_ibadd __P(( nis_name, nis_attr *, uint32_t ,
			nis_object *, nis_object *, struct db_objs * ));
extern db_status db_ibremove __P(( nis_name, nis_attr *, uint32_t ,
			nis_object *, nis_object *, struct db_objs * ));
extern db_status db_ibfirst __P(( nis_name, nis_attr *, uint32_t ,
					struct db_objs * ));
extern db_status db_ibnext __P(( nis_name, netobj *, struct db_objs * ));
extern db_status db_first __P(( nis_name, nis_attr *, uint32_t ,
					struct db_objs * ));
extern db_status db_next __P(( nis_name, netobj *, struct db_objs * ));
extern db_status db_cpdir __P(( nis_name , struct db_objs * ));

/*
 * Special entry mangling routines.
 */
extern struct entryattrs *nis_entry_to_attr __P(( entry_obj * , table_obj * ));
extern nis_object * nis_encap_obj __P(( db_result * , uint32_t ));
extern nis_error nis_mangle_entry __P(( nis_object * , nis_object * ,
					nis_name ));

/*
 * Transaction log routines.
 */
extern nis_error nis_init_log __P(( void ));
extern uint32_t nis_log_find_update __P(( nis_name , uint32_t,
						log_entry ** ));
extern nis_error nis_dump_log __P(( nis_name, uint32_t, log_result ** ));
extern nis_error nis_log_recovery __P(( void ));
extern nis_error nis_log_flush_byxid __P(( uint32_t ));
extern nis_error nis_log_flush_byname __P(( nis_name ));
extern nis_error nis_log_flush_bytime __P(( time_t ));

extern void nis_log_insert_stamp __P(( nis_name, time_t ));
extern void nis_log_free_stamps __P(( void ));
extern void nis_log_commit_stamps __P(( void ));
extern nis_error nis_log_add_entry __P(( log_entry * ));

extern nis_error nis_log_new_xid __P(( nis_name, uint32_t * ));
extern nis_error nis_log_current_xid __P(( uint32_t * ));
extern nis_error nis_log_submit __P((nis_name, nis_object *, nis_attr *,
				     uint32_t, log_entry_t));

extern nis_error nis_log_commit_xid __P(( uint32_t ));
extern nis_error nis_log_abort_xid __P(( uint32_t ));
/*
 * State and stats handling routines
 */

#define NIS_STATE_ACCESS	\
	((NIS_READ_ACC|NIS_MODIFY_ACC) << 16 |	\
	(NIS_READ_ACC|NIS_MODIFY_ACC) << 8)

extern void nis_init_stats __P(( void ));
extern void nis_update_stats __P(( uint32_t, bool_t, uint32_t ));
extern nis_taglist *nis_getstate __P(( nis_taglist * , nis_name ));
extern nis_taglist *nis_setstate __P(( nis_taglist * , nis_name ));

/*
 * Replica support functions
 */

extern nis_error nis_replica_do_updates __P(( void ));
extern nis_error nis_replica_add_update __P(( ping_args * ));
extern nis_error nis_update_cold __P(( void ));
extern nis_error nis_update_replica __P(( nis_name, uint32_t ));
extern nis_error nis_replica_update_root __P(( void ));

/*
 * Master support functions.
 */
extern nis_error nis_master_do_updates __P(( void ));
extern nis_error nis_master_add_update __P(( nis_name ));

/*
 * Master/replica sync support
 */
extern nis_error nis_cptime __P(( nis_name, nis_server *, uint32_t * ));
extern nis_error nis_ping_srv __P(( nis_name, nis_server *, uint32_t ));
extern void nis_sync __P(( void ));

/*
 * Private client lookup routines
 */

extern nis_result *__nis_search __P (( nis_name, uint32_t, uint32_t, nis_attr *, uint32_t, int (*)(nis_name, nis_object *, void *) , void * ));

/*
 * Events
 */
uint32_t nis_event_add_cyclic
	__P(( bool_t (*)(void *, uint32_t, int),
		 void *, void (*)(void *),
		 int, char * ));
uint32_t nis_event_add_oneshot
		__P(( bool_t (*)(void *, uint32_t, int),
			  void *, void (*)(void *),
			  int, char * ));
uint32_t nis_event_add_conditional
			__P(( bool_t (*)(void *, uint32_t, int),
			      void *, void (*)(void *),
			      int, char * ));
uint32_t nis_event_add_sock __P(( bool_t (*)(void *, uint32_t, int),
					  void *, void (*)(void *),
					  int, int, char * ));
uint32_t nis_event_find __P(( char * , void ** ));
void nis_event_del __P(( uint32_t ));
int nis_event_sched __P(( fd_set *, struct timeval * ));
void nis_event_runq __P(( fd_set * ));
void nis_event_init __P(( void ));

extern void __nis_disqualify_server __P(( directory_obj *, nis_name ));
extern void __nis_disqualify_server2 __P(( directory_obj *, nis_name ));
extern nis_error __nis_ping __P(( struct _nis_binding * ));
extern void __nis_flush_one_group __P(( nis_name ));
