/*
 * Copyright (c) 1996, 1997
 *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: nis_log.h,v 1.1 1998/07/08 19:23:20 wpaul Exp wpaul $
 */

#include <sys/cdefs.h>

#ifndef lint
static const char nis_log_h_rcsid[] =
	"@(#) $Id: nis_log.h,v 1.1 1998/07/08 19:23:20 wpaul Exp wpaul $";
#endif

/*
 * Size of log file chunk. Must be multiple of system page size.
 * On FreeBSD, it's 4096.
 */
#define CHUNKSIZE	4096*2

/*
 * Header and spacer structure definitions and magic numbers. The
 * header is the first thing in the log file and contains some log-wide
 * information. The spacers bracket the XDR'd log entries: each spacer
 * contains a pair of offsets that tell the log system where the previous
 * spacer and next spacer are located. The very first spacer and the
 * very last spacer have only one offset (next and previous, respecively).
 *
 * The magic numbers allow us to positively identify the header and spacers
 * in memory. The it is important to note that the offsets are all just
 * relative counts rather that addresses. This is because we have no
 * guarantee that the log will me mmap()ed at the same address each time;
 * the only address we know about for certain is the starting address
 * returned to us by mmap(), so we base everything relative to that.
 * I suppose we could provide pointers and fix them up in nis_log_init(),
 * but we can get by without them, so why bother.
 */
#define HEADER_MAGIC	987654321
#define SPACER_MAGIC	0x8675309

enum log_state {
	LOG_STABLE = 0,
	LOG_UPDATING = 1,
	LOG_SYNCING = 2
};

struct log_header {
	uint32_t		magic;
	enum log_state		state;
	uint32_t		count;
	uint32_t		xid;
	off_t			size;
	off_t			offset;
	time_t			upd_time;
};

struct log_spacer {
	uint32_t		magic;
	uint32_t		xid;
	off_t			prev_offset;
	off_t			next_offset;
};
